@@ -40,6 +40,10 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define RW_SNAPLEN_EVENT 4096
 #define RW_MAX_SNAPLEN (256 * 1024 * 1024)
 #define DPI_LOOKAHED_SIZE 16
+#define PPM_NULL_RDEV MKDEV(1, 3)
+#define PPM_PORT_MYSQL 3306
+#define PPM_PORT_POSTGRES 5432
+#define PPM_PORT_STATSD 8125
 
 /*
  * Global enums
@@ -116,6 +120,10 @@ long ppm_strncpy_from_user(char *to, const char __user *from, unsigned long n);
   #define SYSCALL_TABLE_ID0 __NR_SYSCALL_BASE
 #elif defined CONFIG_X86 || defined CONFIG_SUPERH
   #define SYSCALL_TABLE_ID0 0
+#elif defined CONFIG_PPC64
+  #define SYSCALL_TABLE_ID0 0
+#elif defined CONFIG_S390
+  #define SYSCALL_TABLE_ID0 0
 #endif
 
 #define SYSCALL_TABLE_SIZE 512
@@ -128,7 +136,3 @@ extern const enum ppm_syscall_code g_syscall_code_routing_table[];
 extern const struct syscall_evt_pair g_syscall_ia32_table[];
 extern const enum ppm_syscall_code g_syscall_ia32_code_routing_table[];
 #endif
-
-#define PPM_PORT_MYSQL 3306
-#define PPM_PORT_POSTGRES 5432
-#define PPM_PORT_STATSD 8125
