@@ -284,7 +284,8 @@ scap_t* scap_open_offline_int(const char* fname,
 							  char *error,
 							  proc_entry_callback proc_callback, 
 							  void* proc_callback_context,
-							  bool import_users)
+							  bool import_users,
+							  uint64_t start_offset)
 {
 	scap_t* handle = NULL;
 
@@ -334,6 +335,14 @@ scap_t* scap_open_offline_int(const char* fname,
 		return NULL;
 	}
 
+	//
+	// If this is a merged file, we might have to move the read offset to the next section
+	//
+	if(start_offset != 0)
+	{
+		scap_fseek(handle, start_offset);
+	}
+
 	//
 	// Validate the file and load the non-event blocks
 	//
@@ -368,7 +377,7 @@ scap_t* scap_open_offline_int(const char* fname,
 
 scap_t* scap_open_offline(const char* fname, char *error)
 {
-	return scap_open_offline_int(fname, error, NULL, NULL, true);
+	return scap_open_offline_int(fname, error, NULL, NULL, true, 0);
 }
 
 scap_t* scap_open_live(char *error)
@@ -382,7 +391,8 @@ scap_t* scap_open(scap_open_args args, char *error)
 	{
 		return scap_open_offline_int(args.fname, error, 
 			args.proc_callback, args.proc_callback_context,
-			args.import_users);
+			args.import_users,
+			args.start_offset);
 	}
 	else
 	{
@@ -878,6 +888,33 @@ static int32_t scap_set_dropping_mode(scap_t* handle, int request, uint32_t samp
 }
 #endif
 
+#if defined(HAS_CAPTURE)
+int32_t scap_enable_tracers_capture(scap_t* handle)
+{
+	//	
+	// Not supported for files
+	//
+	if(handle->m_file)
+	{
+		snprintf(handle->m_lasterr,	SCAP_LASTERR_SIZE, "scap_set_inode_of_dev_null not supported on offline captures");
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+	if(handle->m_ndevs)
+	{
+		if(ioctl(handle->m_devs[0].m_fd, PPM_IOCTL_SET_TRACERS_CAPTURE))
+		{
+			snprintf(handle->m_lasterr,	SCAP_LASTERR_SIZE, "%s failed", __FUNCTION__);
+			ASSERT(false);
+			return SCAP_FAILURE;
+		}		
+	}
+
+	return SCAP_SUCCESS;
+}
+#endif
+
 int32_t scap_stop_dropping_mode(scap_t* handle)
 {
 #if !defined(HAS_CAPTURE)
@@ -1237,3 +1274,8 @@ void scap_set_refresh_proc_table_when_saving(scap_t* handle, bool refresh)
 {
 	handle->refresh_proc_table_when_saving = refresh;
 }
+
+uint64_t scap_get_unexpected_block_readsize(scap_t* handle)
+{
+	return handle->m_unexpected_block_readsize;
+}
