@@ -56,6 +56,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	int64_t tmp;
 	uint32_t uid;
 	uint64_t ppid;
+	int64_t sid;
 	uint32_t vmsize_kb;
 	uint32_t vmrss_kb;
 	uint32_t vmswap_kb;
@@ -67,6 +68,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 
 	tinfo->uid = (uint32_t)-1;
 	tinfo->ptid = (uint32_t)-1LL;
+	tinfo->sid = 0;
 	tinfo->vmsize_kb = 0;
 	tinfo->vmrss_kb = 0;
 	tinfo->vmswap_kb = 0;
@@ -195,7 +197,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	{
 		ASSERT(false);
 		fclose(f);
-		return SCAP_FAILURE;		
+		return SCAP_FAILURE;
 	}
 
 	//
@@ -204,7 +206,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	if(sscanf(s + 2, "%c %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64,
 		&tmpc,
 		&tmp,
-		&tmp,
+		&sid,
 		&tmp,
 		&tmp,
 		&tmp,
@@ -220,6 +222,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 
 	tinfo->pfmajor = pfmajor;
 	tinfo->pfminor = pfminor;
+	tinfo->sid = (uint64_t) sid;
 
 	fclose(f);
 	return SCAP_SUCCESS;
@@ -325,7 +328,7 @@ int32_t scap_proc_fill_cgroups(struct scap_threadinfo* tinfo, const char* procdi
 			}
 
 			snprintf(tinfo->cgroups + tinfo->cgroups_len, SCAP_MAX_CGROUPS_SIZE - tinfo->cgroups_len, "%s=%s", token, cgroup);
-			tinfo->cgroups_len += strlen(cgroup) + 1 + strlen(token) + 1;			
+			tinfo->cgroups_len += strlen(cgroup) + 1 + strlen(token) + 1;
 		}
 	}
 
@@ -404,7 +407,7 @@ int32_t scap_getpid_global(scap_t* handle, int64_t* pid)
 	}
 
 	return SCAP_SUCCESS;
-#endif	
+#endif
 }
 
 int32_t scap_proc_fill_root(struct scap_threadinfo* tinfo, const char* procdirname)
@@ -655,7 +658,7 @@ static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parentt
 	{
 		snprintf(error, SCAP_LASTERR_SIZE, "can't fill cgroups for %" PRIu64, tinfo->tid);
 		free(tinfo);
-		return SCAP_FAILURE;	
+		return SCAP_FAILURE;
 	}
 
 	if(scap_get_vtid(handle, tinfo->tid, &tinfo->vtid) == SCAP_FAILURE)
@@ -706,7 +709,7 @@ static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parentt
 	{
 		*procinfo = tinfo;
 	}
-	
+
 	//
 	// Only add fds for processes, not threads
 	//
@@ -970,6 +973,21 @@ void scap_proc_free(scap_t* handle, struct scap_threadinfo* proc)
 	free(proc);
 }
 
+int32_t scap_proc_add(scap_t* handle, uint64_t tid, scap_threadinfo* tinfo)
+{
+	int32_t uth_status = SCAP_SUCCESS;
+
+	HASH_ADD_INT64(handle->m_proclist, tid, tinfo);
+	if(uth_status == SCAP_SUCCESS)
+	{
+		return SCAP_SUCCESS;
+	}
+	else
+	{
+		return SCAP_FAILURE;
+	}
+}
+
 //
 // Internal helper functions to output the process table to screen
 //
