@@ -183,6 +183,7 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 				(sizeof(uint64_t) +	// tid
 				sizeof(uint64_t) +	// pid
 				sizeof(uint64_t) +	// ptid
+				sizeof(uint64_t) +	// sid
 				2 + strnlen(tinfo->comm, SCAP_MAX_PATH_SIZE) +
 				2 + strnlen(tinfo->exe, SCAP_MAX_PATH_SIZE) +
 				2 + tinfo->args_len +
@@ -207,7 +208,7 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 	//
 	// Create the block
 	//
-	bh.block_type = PL_BLOCK_TYPE_V5;
+	bh.block_type = PL_BLOCK_TYPE_V6;
 	bh.block_total_length = scap_normalize_block_len(sizeof(block_header) + totlen + 4);
 
 	if(gzwrite(f, &bh, sizeof(bh)) != sizeof(bh))
@@ -235,6 +236,7 @@ static int32_t scap_write_proclist(scap_t *handle, gzFile f)
 		if(gzwrite(f, &(tinfo->tid), sizeof(uint64_t)) != sizeof(uint64_t) ||
 		        gzwrite(f, &(tinfo->pid), sizeof(uint64_t)) != sizeof(uint64_t) ||
 		        gzwrite(f, &(tinfo->ptid), sizeof(uint64_t)) != sizeof(uint64_t) ||
+		        gzwrite(f, &(tinfo->sid), sizeof(uint64_t)) != sizeof(uint64_t) ||
 		        gzwrite(f, &commlen, sizeof(uint16_t)) != sizeof(uint16_t) ||
 		        gzwrite(f, tinfo->comm, commlen) != commlen ||
 		        gzwrite(f, &exelen, sizeof(uint16_t)) != sizeof(uint16_t) ||
@@ -692,7 +694,7 @@ scap_dumper_t *scap_dump_open(scap_t *handle, const char *fname, compression_mod
 			close(fd);
 		}
 #endif
-		
+
 		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "can't open %s", fname);
 		return NULL;
 	}
@@ -804,7 +806,7 @@ static int32_t scap_read_machine_info(scap_t *handle, gzFile f, uint32_t block_l
 	//
 	// Read the section header block
 	//
-	if(gzread(f, &handle->m_machine_info, sizeof(handle->m_machine_info)) != 
+	if(gzread(f, &handle->m_machine_info, sizeof(handle->m_machine_info)) !=
 		sizeof(handle->m_machine_info))
 	{
 		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "error reading from file (1)");
@@ -841,6 +843,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 	tinfo.cgroups_len = 0;
 	tinfo.filtered_out = 0;
 	tinfo.root[0] = 0;
+	tinfo.sid = -1;
 
 	while(((int32_t)block_length - (int32_t)totreadsize) >= 4)
 	{
@@ -868,6 +871,30 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 
 		totreadsize += readsize;
 
+		switch(block_type)
+		{
+		case PL_BLOCK_TYPE_V1:
+		case PL_BLOCK_TYPE_V1_INT:
+		case PL_BLOCK_TYPE_V2:
+		case PL_BLOCK_TYPE_V2_INT:
+		case PL_BLOCK_TYPE_V3:
+		case PL_BLOCK_TYPE_V3_INT:
+		case PL_BLOCK_TYPE_V4:
+		case PL_BLOCK_TYPE_V5:
+			break;
+		case PL_BLOCK_TYPE_V6:
+			readsize = gzread(f, &(tinfo.sid), sizeof(uint64_t));
+			CHECK_READ_SIZE(readsize, sizeof(uint64_t));
+
+			totreadsize += readsize;
+			break;
+
+		default:
+			snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted process block type (fd1)");
+			ASSERT(false);
+			return SCAP_FAILURE;
+		}
+
 		//
 		// comm
 		//
@@ -1000,6 +1027,7 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 		case PL_BLOCK_TYPE_V3_INT:
 		case PL_BLOCK_TYPE_V4:
 		case PL_BLOCK_TYPE_V5:
+		case PL_BLOCK_TYPE_V6:
 			//
 			// vmsize_kb
 			//
@@ -1043,7 +1071,8 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 			if(block_type == PL_BLOCK_TYPE_V3 ||
 				block_type == PL_BLOCK_TYPE_V3_INT ||
 				block_type == PL_BLOCK_TYPE_V4 ||
-				block_type == PL_BLOCK_TYPE_V5)
+				block_type == PL_BLOCK_TYPE_V5 ||
+				block_type == PL_BLOCK_TYPE_V6)
 			{
 				//
 				// env
@@ -1069,7 +1098,9 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 				totreadsize += readsize;
 			}
 
-			if(block_type == PL_BLOCK_TYPE_V4 || block_type == PL_BLOCK_TYPE_V5)
+			if(block_type == PL_BLOCK_TYPE_V4 ||
+			   block_type == PL_BLOCK_TYPE_V5 ||
+			   block_type == PL_BLOCK_TYPE_V6)
 			{
 				//
 				// vtid
@@ -1107,7 +1138,8 @@ static int32_t scap_read_proclist(scap_t *handle, gzFile f, uint32_t block_lengt
 
 				totreadsize += readsize;
 
-				if(block_type == PL_BLOCK_TYPE_V5)
+				if(block_type == PL_BLOCK_TYPE_V5 ||
+				   block_type == PL_BLOCK_TYPE_V6)
 				{
 					readsize = gzread(f, &(stlen), sizeof(uint16_t));
 					CHECK_READ_SIZE(readsize, sizeof(uint16_t));
@@ -1897,6 +1929,7 @@ int32_t scap_read_init(scap_t *handle, gzFile f)
 		case PL_BLOCK_TYPE_V3:
 		case PL_BLOCK_TYPE_V4:
 		case PL_BLOCK_TYPE_V5:
+		case PL_BLOCK_TYPE_V6:
 		case PL_BLOCK_TYPE_V1_INT:
 		case PL_BLOCK_TYPE_V2_INT:
 		case PL_BLOCK_TYPE_V3_INT:
@@ -1996,13 +2029,13 @@ int32_t scap_read_init(scap_t *handle, gzFile f)
 
 	if(!found_ul)
 	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find user list block.");			
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find user list block.");
 		return SCAP_FAILURE;
 	}
 
 	if(!found_il)
 	{
-		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find interface list block.");			
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "corrupted input file. Can't find interface list block.");
 		return SCAP_FAILURE;
 	}
 
@@ -2049,12 +2082,13 @@ int32_t scap_next_offline(scap_t *handle, OUT scap_evt **pevent, OUT uint16_t *p
 		}
 	}
 
-	if(bh.block_type != EV_BLOCK_TYPE && 
+	if(bh.block_type != EV_BLOCK_TYPE &&
 		bh.block_type != EV_BLOCK_TYPE_INT &&
 		bh.block_type != EVF_BLOCK_TYPE)
 	{
 		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "unexpected block type %u", (uint32_t)bh.block_type);
-		return SCAP_FAILURE;
+		handle->m_unexpected_block_readsize = readsize;
+		return SCAP_UNEXPECTED_BLOCK;
 	}
 
 	if(bh.block_total_length < sizeof(bh) + sizeof(struct ppm_evt_hdr) + 4)
@@ -2067,6 +2101,13 @@ int32_t scap_next_offline(scap_t *handle, OUT scap_evt **pevent, OUT uint16_t *p
 	// Read the event
 	//
 	readlen = bh.block_total_length - sizeof(bh);
+	if (readlen > FILE_READ_BUF_SIZE) {
+		snprintf(handle->m_lasterr, SCAP_LASTERR_SIZE, "event block length %u greater than read buffer size %u",
+			 readlen,
+			 FILE_READ_BUF_SIZE);
+		return SCAP_FAILURE;
+	}
+
 	readsize = gzread(f, handle->m_file_evt_buf, readlen);
 	CHECK_READ_SIZE(readsize, readlen);
 
