@@ -114,7 +114,7 @@ uint32_t lua_cbacks::rawval_to_lua_stack(lua_State *ls, uint8_t* rawval, const f
 		case PT_BYTEBUF:
 			if(rawval[len] == 0)
 			{
-				lua_pushstring(ls, (char*)rawval);
+				lua_pushlstring(ls, (char*)rawval, len);
 				return 1;
 			}
 			else
@@ -128,7 +128,7 @@ uint32_t lua_cbacks::rawval_to_lua_stack(lua_State *ls, uint8_t* rawval, const f
 
 				memcpy(ch->m_lua_fld_storage, rawval, max_len);
 				ch->m_lua_fld_storage[max_len] = 0;
-				lua_pushstring(ls, (char*)ch->m_lua_fld_storage);
+				lua_pushlstring(ls, (char*)ch->m_lua_fld_storage, max_len);
 				return 1;
 			}
 		case PT_SOCKADDR:
@@ -284,7 +284,7 @@ int lua_cbacks::request_field(lua_State *ls)
 		throw sinsp_exception("chisel error");
 	}
 
-	chk->parse_field_name(fld, true);
+	chk->parse_field_name(fld, true, false);
 
 	lua_pushlightuserdata(ls, chk);
 
@@ -445,6 +445,14 @@ int lua_cbacks::set_output_format(lua_State *ls)
 	{
 		ch->m_inspector->set_buffer_format(sinsp_evt::PF_EOLS);
 	}
+	else if(string(fmt) == "base64")
+	{
+		ch->m_inspector->set_buffer_format(sinsp_evt::PF_BASE64);
+	}
+	else if(string(fmt) == "jsonbase64")
+	{
+		ch->m_inspector->set_buffer_format(sinsp_evt::PF_JSONBASE64);
+	}
 	else
 	{
 		string err = "invalid set_output_format value in chisel " + ch->m_filename;
