@@ -54,6 +54,8 @@ public:
 	static int set_interval_s(lua_State *ls);
 	static int exec(lua_State *ls);
 	static int log(lua_State *ls);
+	static int udp_setpeername(lua_State *ls);
+	static int udp_send(lua_State *ls);
 #ifdef HAS_ANALYZER
 	static int push_metric(lua_State *ls);
 #endif
