@@ -25,6 +25,59 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "sinsp.h"
 #include "sinsp_int.h"
 #include "container.h"
+#include "utils.h"
+
+void sinsp_container_info::parse_json_mounts(const Json::Value &mnt_obj, vector<sinsp_container_info::container_mount_info> &mounts)
+{
+	if(!mnt_obj.isNull() && mnt_obj.isArray())
+	{
+		for(uint32_t i=0; i<mnt_obj.size(); i++)
+		{
+			const Json::Value &mount = mnt_obj[i];
+			mounts.emplace_back(mount["Source"], mount["Destination"],
+					    mount["Mode"], mount["RW"],
+					    mount["Propagation"]);
+		}
+	}
+}
+
+sinsp_container_info::container_mount_info *sinsp_container_info::mount_by_idx(uint32_t idx)
+{
+	if (idx >= m_mounts.size())
+	{
+		return NULL;
+	}
+
+	return &(m_mounts[idx]);
+}
+
+sinsp_container_info::container_mount_info *sinsp_container_info::mount_by_source(std::string &source)
+{
+	// note: linear search
+	for (auto &mntinfo :m_mounts)
+	{
+		if(sinsp_utils::glob_match(source.c_str(), mntinfo.m_source.c_str()))
+		{
+			return &mntinfo;
+		}
+	}
+
+	return NULL;
+}
+
+sinsp_container_info::container_mount_info *sinsp_container_info::mount_by_dest(std::string &dest)
+{
+	// note: linear search
+	for (auto &mntinfo :m_mounts)
+	{
+		if(sinsp_utils::glob_match(dest.c_str(), mntinfo.m_dest.c_str()))
+		{
+			return &mntinfo;
+		}
+	}
+
+	return NULL;
+}
 
 sinsp_container_manager::sinsp_container_manager(sinsp* inspector) :
 	m_inspector(inspector),
@@ -41,7 +94,7 @@ bool sinsp_container_manager::remove_inactive_containers()
 		m_last_flush_time_ns = m_inspector->m_lastevent_ts - m_inspector->m_inactive_container_scan_time_ns + 30 * ONE_SECOND_IN_NS;
 	}
 
-	if(m_inspector->m_lastevent_ts > 
+	if(m_inspector->m_lastevent_ts >
 		m_last_flush_time_ns + m_inspector->m_inactive_thread_scan_time_ns)
 	{
 		res = true;
@@ -151,7 +204,7 @@ bool sinsp_container_manager::set_mesos_task_id(sinsp_container_info* container,
 			else
 			{
 				g_logger.log("Mesos task ID not found for Mesos container [" + container->m_id + "],"
-							 "thread [" + std::to_string(tinfo->m_tid) + ']', sinsp_logger::SEV_WARNING);
+							 "thread [" + std::to_string(tinfo->m_tid) + ']', sinsp_logger::SEV_DEBUG);
 			}
 		}
 	}
@@ -187,7 +240,7 @@ bool sinsp_container_manager::resolve_container(sinsp_threadinfo* tinfo, bool qu
 		if(pos != string::npos)
 		{
 			if(cgroup.length() - pos - 1 == 64 &&
-				cgroup.find_first_not_of("0123456789abcdefABCDEF", pos + 1) == string::npos) 
+				cgroup.find_first_not_of("0123456789abcdefABCDEF", pos + 1) == string::npos)
 			{
 				container_info.m_type = CT_DOCKER;
 				container_info.m_id = cgroup.substr(pos + 1, 12);
@@ -295,7 +348,7 @@ bool sinsp_container_manager::resolve_container(sinsp_threadinfo* tinfo, bool qu
 		static const string COREOS_PODID_VAR = "container_uuid=";
 
 		auto prefix = tinfo->m_root.find(COREOS_PREFIX);
-		if(prefix != string::npos)
+		if(prefix == 0)
 		{
 			auto suffix = tinfo->m_root.find(COREOS_APP_SUFFIX, prefix);
 			if(suffix != string::npos)
@@ -330,7 +383,7 @@ bool sinsp_container_manager::resolve_container(sinsp_threadinfo* tinfo, bool qu
 			static const string FLY_APP_SUFFIX = "/rootfs";
 
 			auto prefix = tinfo->m_root.find(FLY_PREFIX);
-			if(prefix != string::npos)
+			if(prefix == 0)
 			{
 				auto podid_suffix = tinfo->m_root.find(FLY_PODID_SUFFIX, prefix+FLY_PREFIX.size());
 				if(podid_suffix != string::npos)
@@ -407,6 +460,31 @@ string sinsp_container_manager::container_to_json(const sinsp_container_info& co
 	container["type"] = container_info.m_type;
 	container["name"] = container_info.m_name;
 	container["image"] = container_info.m_image;
+	container["imageid"] = container_info.m_imageid;
+	container["privileged"] = container_info.m_privileged;
+
+	Json::Value mounts = Json::arrayValue;
+
+	for (auto &mntinfo : container_info.m_mounts)
+	{
+		Json::Value mount;
+
+		mount["Source"] = mntinfo.m_source;
+		mount["Destination"] = mntinfo.m_dest;
+		mount["Mode"] = mntinfo.m_mode;
+		mount["RW"] = mntinfo.m_rdwr;
+		mount["Propagation"] = mntinfo.m_propagation;
+
+		mounts.append(mount);
+	}
+
+	container["Mounts"] = mounts;
+
+	char addrbuff[100];
+	uint32_t iph = ntohl(container_info.m_container_ip);
+	inet_ntop(AF_INET, &iph, addrbuff, sizeof(addrbuff));
+	container["ip"] = addrbuff;
+
 	if(!container_info.m_mesos_task_id.empty())
 	{
 		container["mesos_task_id"] = container_info.m_mesos_task_id;
@@ -515,6 +593,14 @@ bool sinsp_container_manager::parse_docker(sinsp_container_info* container)
 	const Json::Value& config_obj = root["Config"];
 
 	container->m_image = config_obj["Image"].asString();
+
+	string imgstr = root["Image"].asString();
+	size_t cpos = imgstr.find(":");
+	if(cpos != string::npos)
+	{
+		container->m_imageid = imgstr.substr(cpos + 1);
+	}
+
 	container->m_name = root["Name"].asString();
 
 	if(!container->m_name.empty() && container->m_name[0] == '/')
@@ -546,7 +632,7 @@ bool sinsp_container_manager::parse_docker(sinsp_container_info* container)
 		if(v.isArray())
 		{
 			for(uint32_t j = 0; j < v.size(); ++j)
-			{	
+			{
 				sinsp_container_info::container_port_mapping port_mapping;
 
 				ip = v[j]["HostIp"].asString();
@@ -574,14 +660,14 @@ bool sinsp_container_manager::parse_docker(sinsp_container_info* container)
 	}
 
 	const Json::Value& env_vars = config_obj["Env"];
-	string mesos_task_id = get_mesos_task_id(env_vars, "MESOS_TASK_ID");
+	string mesos_task_id = get_docker_env(env_vars, "MESOS_TASK_ID");
 	if(mesos_task_id.empty())
 	{
-		mesos_task_id = get_mesos_task_id(env_vars, "mesos_task_id");
+		mesos_task_id = get_docker_env(env_vars, "mesos_task_id");
 	}
 	if(mesos_task_id.empty())
 	{
-		mesos_task_id = get_mesos_task_id(env_vars, "MESOS_EXECUTOR_ID");
+		mesos_task_id = get_docker_env(env_vars, "MESOS_EXECUTOR_ID");
 	}
 	if(!mesos_task_id.empty())
 	{
@@ -589,20 +675,45 @@ bool sinsp_container_manager::parse_docker(sinsp_container_info* container)
 		g_logger.log("Mesos Docker container: [" + root["Id"].asString() + "], Mesos task ID: [" + container->m_mesos_task_id + ']', sinsp_logger::SEV_DEBUG);
 	}
 
+	const auto& host_config_obj = root["HostConfig"];
+	container->m_memory_limit = host_config_obj["Memory"].asInt64();
+	container->m_swap_limit = host_config_obj["MemorySwap"].asInt64();
+	const auto cpu_shares = host_config_obj["CpuShares"].asInt64();
+	if(cpu_shares > 0)
+	{
+		container->m_cpu_shares = cpu_shares;
+	}
+	container->m_cpu_quota = host_config_obj["CpuQuota"].asInt64();
+	const auto cpu_period = host_config_obj["CpuPeriod"].asInt64();
+	if(cpu_period > 0)
+	{
+		container->m_cpu_period = cpu_period;
+	}
+	const Json::Value &privileged = host_config_obj["Privileged"];
+	if(!privileged.isNull() && privileged.isBool())
+	{
+		container->m_privileged = privileged.asBool();
+	}
+
+	sinsp_container_info::parse_json_mounts(root["Mounts"], container->m_mounts);
+
+#ifdef HAS_ANALYZER
+	container->m_sysdig_agent_conf = get_docker_env(env_vars, "SYSDIG_AGENT_CONF");
+#endif
 	return true;
 }
 
-string sinsp_container_manager::get_mesos_task_id(const Json::Value& env_vars, const string& mti)
+string sinsp_container_manager::get_docker_env(const Json::Value &env_vars, const string &mti)
 {
-	string mesos_task_id;
+	string ret;
 	for(const auto& env_var : env_vars)
 	{
 		if(env_var.isString())
 		{
-			mesos_task_id = env_var.asString();
-			if((mesos_task_id.length() > (mti.length() + 1)) && (mesos_task_id.substr(0, mti.length()) == mti))
+			ret = env_var.asString();
+			if((ret.length() > (mti.length() + 1)) && (ret.substr(0, mti.length()) == mti))
 			{
-				return mesos_task_id.substr(mti.length() + 1);
+				return ret.substr(mti.length() + 1);
 			}
 		}
 	}
@@ -616,7 +727,6 @@ bool sinsp_container_manager::parse_rkt(sinsp_container_info *container,
 	Json::Reader reader;
 	Json::Value jroot;
 
-	unordered_map<string, uint32_t> image_ports;
 	char image_manifest_path[SCAP_MAX_PATH_SIZE];
 	snprintf(image_manifest_path, sizeof(image_manifest_path), "%s/var/lib/rkt/pods/run/%s/appsinfo/%s/manifest", scap_get_host_root(), podid.c_str(), appname.c_str());
 	ifstream image_manifest(image_manifest_path);
@@ -632,10 +742,6 @@ bool sinsp_container_manager::parse_rkt(sinsp_container_info *container,
 		{
 			container->m_image += ":" + version_label_it->second;
 		}
-		for(const auto& image_port : jroot["app"]["ports"])
-		{
-			image_ports.emplace(image_port["name"].asString(), image_port["port"].asUInt());
-		}
 		ret = true;
 	}
 
@@ -644,7 +750,7 @@ bool sinsp_container_manager::parse_rkt(sinsp_container_info *container,
 	ifstream net_info(net_info_path);
 	if(reader.parse(net_info, jroot) && jroot.size() > 0)
 	{
-		auto first_net = jroot[0];
+		const auto& first_net = jroot[0];
 		if(inet_pton(AF_INET, first_net["ip"].asCString(), &container->m_container_ip) == -1)
 		{
 			ASSERT(false);
@@ -655,16 +761,29 @@ bool sinsp_container_manager::parse_rkt(sinsp_container_info *container,
 	char pod_manifest_path[SCAP_MAX_PATH_SIZE];
 	snprintf(pod_manifest_path, sizeof(pod_manifest_path), "%s/var/lib/rkt/pods/run/%s/pod", scap_get_host_root(), podid.c_str());
 	ifstream pod_manifest(pod_manifest_path);
+	unordered_map<string, uint32_t> image_ports;
 	if(reader.parse(pod_manifest, jroot) && jroot.size() > 0)
 	{
+		for(const auto& japp : jroot["apps"])
+		{
+			if (japp["name"].asString() == appname)
+			{
+				for(const auto& image_port : japp["app"]["ports"])
+				{
+					image_ports[image_port["name"].asString()] = image_port["port"].asUInt();
+				}
+				break;
+			}
+		}
 		for(const auto& jport : jroot["ports"])
 		{
 			auto host_port = jport["hostPort"].asUInt();
-			if(host_port > 0)
+			auto container_port_it = image_ports.find(jport["name"].asString());
+			if(host_port > 0 && container_port_it != image_ports.end())
 			{
 				sinsp_container_info::container_port_mapping port_mapping;
 				port_mapping.m_host_port = host_port;
-				port_mapping.m_container_port = image_ports.at(jport["name"].asString());
+				port_mapping.m_container_port = container_port_it->second;
 				container->m_port_mappings.emplace_back(move(port_mapping));
 			}
 		}
@@ -681,6 +800,11 @@ const unordered_map<string, sinsp_container_info>* sinsp_container_manager::get_
 void sinsp_container_manager::add_container(const sinsp_container_info& container_info)
 {
 	m_containers[container_info.m_id] = container_info;
+
+	if(m_inspector->m_parser->m_fd_listener)
+	{
+		m_inspector->m_parser->m_fd_listener->on_new_container(container_info);
+	}
 }
 
 void sinsp_container_manager::dump_containers(scap_dumper_t* dumper)
