@@ -95,11 +95,6 @@ int32_t sinsp_evt::get_check_id()
 	return m_check_id;
 }
 
-uint64_t sinsp_evt::get_ts()
-{
-	return m_pevt->ts;
-}
-
 uint32_t sinsp_evt::get_dump_flags()
 {
 	return scap_event_get_dump_flags(m_inspector->m_h);
@@ -140,11 +135,6 @@ sinsp_threadinfo* sinsp_evt::get_thread_info(bool query_os_if_not_found)
 	return m_inspector->get_thread(m_pevt->tid, query_os_if_not_found, false);
 }
 
-sinsp_fdinfo_t* sinsp_evt::get_fd_info()
-{
-	return m_fdinfo;
-}
-
 int64_t sinsp_evt::get_fd_num()
 {
 	if(m_fdinfo)
@@ -590,7 +580,7 @@ int sinsp_evt::render_fd_json(Json::Value *ret, int64_t fd, const char** resolve
 			//
 			string sanitized_str = fdinfo->m_name;
 
-			sanitized_str.erase(remove_if(sanitized_str.begin(), sanitized_str.end(), g_invalidchar()), sanitized_str.end());
+			sanitize_string(sanitized_str);
 
 			(*ret)["typechar"] = typestr;
 			(*ret)["name"] = sanitized_str;
@@ -676,7 +666,7 @@ char* sinsp_evt::render_fd(int64_t fd, const char** resolved_str, sinsp_evt::par
 			//
 			string sanitized_str = fdinfo->m_name;
 
-			sanitized_str.erase(remove_if(sanitized_str.begin(), sanitized_str.end(), g_invalidchar()), sanitized_str.end());
+			sanitize_string(sanitized_str);
 
 			//
 			// Make sure the string will fit
@@ -860,14 +850,14 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 				         m_resolved_paramstr_storage.size(),
 				         "%s", errstr.c_str());
 			}
-		} 
+		}
 		ret = (Json::Value::Int64)val;
 	}
 	break;
 
 	case PT_FD:
 		{
-			// We use the string extractor to get 
+			// We use the string extractor to get
 			// the resolved path, but use our routine
 			// to get the actual value to return
 			ASSERT(payload_len == sizeof(int64_t));
@@ -886,7 +876,7 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 	case PT_SOCKADDR:
 		if(payload_len == 0)
 		{
-			ret = Json::Value::nullRef;
+			ret = Json::nullValue;
 			break;
 		}
 		else if(payload[0] == AF_UNIX)
@@ -896,8 +886,8 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 			//
 			// Sanitize the file string.
 			//
-            string sanitized_str = payload + 1;
-            sanitized_str.erase(remove_if(sanitized_str.begin(), sanitized_str.end(), g_invalidchar()), sanitized_str.end());
+			string sanitized_str = payload + 1;
+			sanitize_string(sanitized_str);
 
 			ret = sanitized_str;
 		}
@@ -935,7 +925,7 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 	case PT_SOCKTUPLE:
 		if(payload_len == 0)
 		{
-			ret = Json::Value::nullRef;
+			ret = Json::nullValue;
 			break;
 		}
 
@@ -1039,7 +1029,7 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 					char dststr[INET6_ADDRSTRLEN];
 
 					if(inet_ntop(AF_INET6, sip6, srcstr, sizeof(srcstr)) &&
-						inet_ntop(AF_INET6, sip6, dststr, sizeof(dststr)))
+						inet_ntop(AF_INET6, dip6, dststr, sizeof(dststr)))
 					{
 						Json::Value source;
 						Json::Value dest;
@@ -1069,7 +1059,7 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 			// Sanitize the file string.
 			//
             string sanitized_str = payload + 17;
-            sanitized_str.erase(remove_if(sanitized_str.begin(), sanitized_str.end(), g_invalidchar()), sanitized_str.end());
+	    sanitize_string(sanitized_str);
 
 			snprintf(&m_paramstr_storage[0],
 				m_paramstr_storage.size(),
@@ -1248,7 +1238,7 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 	case PT_CHARBUF_PAIR_ARRAY:
 	{
 		ASSERT(param->m_len == sizeof(uint64_t));
-		pair<vector<char*>*, vector<char*>*>* pairs = 
+		pair<vector<char*>*, vector<char*>*>* pairs =
 			(pair<vector<char*>*, vector<char*>*>*)*(uint64_t *)param->m_val;
 		ASSERT(pairs->first->size() == pairs->second->size());
 
@@ -1268,8 +1258,8 @@ Json::Value sinsp_evt::get_param_as_json(uint32_t id, OUT const char** resolved_
 			char* dst = &m_paramstr_storage[0];
 			char* dstend = &m_paramstr_storage[0] + m_paramstr_storage.size() - 2;
 
-			for(it1 = itbeg1 = pairs->first->begin(), it2 = itbeg2 = pairs->second->begin(); 
-			it1 != pairs->first->end(); 
+			for(it1 = itbeg1 = pairs->first->begin(), it2 = itbeg2 = pairs->second->begin();
+			it1 != pairs->first->end();
 				++it1, ++it2)
 			{
 				char* src = *it1;
@@ -1612,7 +1602,7 @@ const char* sinsp_evt::get_param_as_str(uint32_t id, OUT const char** resolved_s
 			}
 
 			ASSERT(m_inspector != NULL);
-			if(m_inspector->m_max_evt_output_len != 0 && 
+			if(m_inspector->m_max_evt_output_len != 0 &&
 				blen > m_inspector->m_max_evt_output_len &&
 				fmt == PF_NORMAL)
 			{
@@ -1653,7 +1643,7 @@ const char* sinsp_evt::get_param_as_str(uint32_t id, OUT const char** resolved_s
 			// Sanitize the file string.
 			//
             string sanitized_str = payload + 1;
-            sanitized_str.erase(remove_if(sanitized_str.begin(), sanitized_str.end(), g_invalidchar()), sanitized_str.end());
+	    sanitize_string(sanitized_str);
 
 			snprintf(&m_paramstr_storage[0],
 				m_paramstr_storage.size(),
@@ -1753,7 +1743,7 @@ const char* sinsp_evt::get_param_as_str(uint32_t id, OUT const char** resolved_s
 					char srcstr[INET6_ADDRSTRLEN];
 					char dststr[INET6_ADDRSTRLEN];
 					if(inet_ntop(AF_INET6, sip6, srcstr, sizeof(srcstr)) &&
-						inet_ntop(AF_INET6, sip6, dststr, sizeof(dststr)))
+						inet_ntop(AF_INET6, dip6, dststr, sizeof(dststr)))
 					{
 						snprintf(&m_paramstr_storage[0],
 								 m_paramstr_storage.size(),
@@ -1780,7 +1770,7 @@ const char* sinsp_evt::get_param_as_str(uint32_t id, OUT const char** resolved_s
 			// Sanitize the file string.
 			//
             string sanitized_str = payload + 17;
-            sanitized_str.erase(remove_if(sanitized_str.begin(), sanitized_str.end(), g_invalidchar()), sanitized_str.end());
+	    sanitize_string(sanitized_str);
 
 			snprintf(&m_paramstr_storage[0],
 				m_paramstr_storage.size(),
@@ -2105,7 +2095,7 @@ const char* sinsp_evt::get_param_as_str(uint32_t id, OUT const char** resolved_s
 	case PT_CHARBUF_PAIR_ARRAY:
 	{
 		ASSERT(param->m_len == sizeof(uint64_t));
-		pair<vector<char*>*, vector<char*>*>* pairs = 
+		pair<vector<char*>*, vector<char*>*>* pairs =
 			(pair<vector<char*>*, vector<char*>*>*)*(uint64_t *)param->m_val;
 
 		m_paramstr_storage[0] = 0;
@@ -2130,8 +2120,8 @@ const char* sinsp_evt::get_param_as_str(uint32_t id, OUT const char** resolved_s
 			char* dst = &m_paramstr_storage[0];
 			char* dstend = &m_paramstr_storage[0] + m_paramstr_storage.size() - 2;
 
-			for(it1 = itbeg1 = pairs->first->begin(), it2 = itbeg2 = pairs->second->begin(); 
-			it1 != pairs->first->end(); 
+			for(it1 = itbeg1 = pairs->first->begin(), it2 = itbeg2 = pairs->second->begin();
+			it1 != pairs->first->end();
 				++it1, ++it2)
 			{
 				char* src = *it1;
@@ -2450,7 +2440,7 @@ scap_dump_flags sinsp_evt::get_dump_flags(OUT bool* should_drop)
 	{
 		dflags |= SCAP_DF_TRACER;
 	}
-	
+
 	return (scap_dump_flags)dflags;
 }
 #endif
