@@ -128,8 +128,8 @@ void sinsp_evt_formatter::set_format(const string& fmt)
 				}
 			}
 
-			sinsp_filter_check* chk = g_filterlist.new_filter_check_from_fldname(string(cfmt + j + 1), 
-				m_inspector, 
+			sinsp_filter_check* chk = g_filterlist.new_filter_check_from_fldname(string(cfmt + j + 1),
+				m_inspector,
 				false);
 
 			if(chk == NULL)
@@ -139,7 +139,7 @@ void sinsp_evt_formatter::set_format(const string& fmt)
 
 			m_chks_to_free.push_back(chk);
 
-			j += chk->parse_field_name(cfmt + j + 1, true);
+			j += chk->parse_field_name(cfmt + j + 1, true, false);
 			ASSERT(j <= lfmt.length());
 
 			m_tokens.push_back(chk);
@@ -151,7 +151,9 @@ void sinsp_evt_formatter::set_format(const string& fmt)
 
 	if(last_nontoken_str_start != j)
 	{
-		m_tokens.push_back(new rawstring_check(lfmt.substr(last_nontoken_str_start, j - last_nontoken_str_start)));
+		sinsp_filter_check * chk = new rawstring_check(lfmt.substr(last_nontoken_str_start, j - last_nontoken_str_start));
+		m_tokens.push_back(chk);
+		m_chks_to_free.push_back(chk);
 		m_tokenlens.push_back(0);
 	}
 }
@@ -198,7 +200,7 @@ bool sinsp_evt_formatter::tostring(sinsp_evt* evt, OUT string* res)
 				continue;
 			}
 
-			if(json_value == Json::Value::nullRef && m_require_all_values)
+			if(json_value == Json::nullValue && m_require_all_values)
 			{
 				retval = false;
 				continue;
@@ -206,12 +208,12 @@ bool sinsp_evt_formatter::tostring(sinsp_evt* evt, OUT string* res)
 
 			fi = m_tokens[j]->get_field_info();
 
-			if(fi && fi->m_name) 
+			if(fi)
 			{
 				m_root[fi->m_name] = m_tokens[j]->tojson(evt);
-			} 
-		} 
-		else 
+			}
+		}
+		else
 		{
 			char* str = m_tokens[j]->tostring(evt);
 
@@ -220,14 +222,14 @@ bool sinsp_evt_formatter::tostring(sinsp_evt* evt, OUT string* res)
 				continue;
 			}
 
-			if(str == NULL) 
+			if(str == NULL)
 			{
 				if(m_require_all_values)
 				{
 					retval = false;
 					continue;
 				}
-				else 
+				else
 				{
 					str = (char*)"<NA>";
 				}
@@ -254,13 +256,13 @@ bool sinsp_evt_formatter::tostring(sinsp_evt* evt, OUT string* res)
 	   || m_inspector->get_buffer_format() == sinsp_evt::PF_JSONHEXASCII
 	   || m_inspector->get_buffer_format() == sinsp_evt::PF_JSONBASE64)
 	{
-		if(m_first) 
+		if(m_first)
 		{
 			// Give it the opening stanza of a JSON array
 			(*res) = '[';
 			m_first = false;
-		} 
-		else 
+		}
+		else
 		{
 			// Otherwise say this is another object in an
 			// existing JSON array
