@@ -38,6 +38,9 @@ enum cmpop {
 	CO_IN = 8,
 	CO_EXISTS = 9,
 	CO_ICONTAINS = 10,
+	CO_STARTSWITH = 11,
+	CO_GLOB = 12,
+	CO_PMATCH = 13
 };
 
 enum boolop
@@ -155,6 +158,50 @@ private:
 	friend class sinsp_evt_formatter;
 };
 
+/*!
+  \brief This class represents a filter optimized using event
+  types. It actually consists of collections of sinsp_filter objects
+  grouped by event type.
+*/
+
+class SINSP_PUBLIC sinsp_evttype_filter
+{
+public:
+	sinsp_evttype_filter();
+	virtual ~sinsp_evttype_filter();
+
+	void add(std::string &name,
+		 list<uint32_t> &evttypes,
+		 sinsp_filter* filter);
+
+	void enable(std::string &pattern, bool enabled);
+
+	bool run(sinsp_evt *evt);
+
+private:
+
+	struct filter_wrapper {
+		sinsp_filter *filter;
+		list<uint32_t> evttypes;
+		bool enabled;
+	};
+
+	// Maps from event type to filter. There can be multiple
+	// filters per event type.
+	list<filter_wrapper *> *m_filter_by_evttype[PPM_EVENT_MAX];
+
+	// It's possible to add an event type filter with an empty
+	// list of event types, meaning it should run for all event
+	// types.
+	list<filter_wrapper *> m_catchall_evttype_filters;
+
+	// This holds all the filters in
+	// m_filter_by_evttype/m_catchall_evttype_filters, so they can
+	// be cleaned up.
+
+	map<std::string,filter_wrapper *> m_evttype_filters;
+};
+
 /*@}*/
 
 #endif // HAS_FILTERING
