@@ -35,12 +35,12 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "sinsp.h"
 #include "sinsp_int.h"
 #include "../../driver/ppm_ringbuffer.h"
+#include "tracers.h"
 #include "parsers.h"
 #include "sinsp_errno.h"
 #include "filter.h"
 #include "filterchecks.h"
 #include "protodecoder.h"
-#include "tracers.h"
 #ifdef HAS_ANALYZER
 #include "analyzer_int.h"
 #include "analyzer_thread.h"
@@ -63,6 +63,7 @@ sinsp_parser::sinsp_parser(sinsp *inspector) :
 
 	sinsp_tracerparser p(inspector);
 	p.test();
+	m_drop_event_flags = EF_NONE;
 }
 #else
 sinsp_parser::sinsp_parser(sinsp *inspector) :
@@ -80,6 +81,7 @@ sinsp_parser::sinsp_parser(sinsp *inspector) :
 
 	init_metaevt(m_k8s_metaevents_state, PPME_K8S_E, SP_EVT_BUF_SIZE);
 	init_metaevt(m_mesos_metaevents_state, PPME_MESOS_E, SP_EVT_BUF_SIZE);
+	m_drop_event_flags = EF_NONE;
 }
 #endif
 
@@ -145,8 +147,8 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 #if defined(HAS_CAPTURE)
 	if(is_live && !m_inspector->is_debug_enabled())
 	{
-		if(evt->get_tid() == m_inspector->m_sysdig_pid && 
-			etype != PPME_SCHEDSWITCH_1_E && 
+		if(evt->get_tid() == m_inspector->m_sysdig_pid &&
+			etype != PPME_SCHEDSWITCH_1_E &&
 			etype != PPME_SCHEDSWITCH_6_E &&
 			etype != PPME_DROP_E &&
 			etype != PPME_DROP_X &&
@@ -160,20 +162,42 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	}
 #endif
 
+		if (m_drop_event_flags)
+		{
+			enum ppm_event_flags flags;
+			uint16_t etype = evt->m_pevt->type;
+			if(etype == PPME_GENERIC_E || etype == PPME_GENERIC_X)
+			{
+				sinsp_evt_param *parinfo = evt->get_param(0);
+				uint16_t evid = *(uint16_t *)parinfo->m_val;
+				flags = g_infotables.m_syscall_info_table[evid].flags;
+			}
+			else
+			{
+				flags = evt->get_info_flags();
+			}
+
+			if (flags & m_drop_event_flags)
+			{
+				evt->m_filtered_out = true;
+				return;
+			}
+		}
+
 	//
 	// Filtering
 	//
 #if defined(HAS_FILTERING) && defined(HAS_CAPTURE_FILTERING)
 	bool do_filter_later = false;
 
-	if(m_inspector->m_filter)
+	if(m_inspector->m_filter || m_inspector->m_evttype_filter)
 	{
 		ppm_event_flags eflags = evt->get_info_flags();
 
 		if(etype == PPME_SYSCALL_WRITE_X)
 		{
 			//
-			// Check if this is a user event
+			// Check if this is a tracer
 			//
 			sinsp_fdinfo_t* fdinfo = evt->m_fdinfo;
 
@@ -183,7 +207,7 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 				evt->m_fdinfo = fdinfo;
 			}
 
-			if(fdinfo && fdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD)
+			if(fdinfo && (fdinfo->m_flags & (sinsp_fdinfo_t::FLAGS_IS_TRACER_FD | sinsp_fdinfo_t::FLAGS_IS_TRACER_FILE)))
 			{
 				eflags = (ppm_event_flags)(((uint64_t)eflags) | EF_MODIFIES_STATE);
 			}
@@ -206,7 +230,7 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 		}
 		else
 		{
-			if(m_inspector->m_filter->run(evt) == false)
+			if(m_inspector->run_filters_on_evt(evt) == false)
 			{
 				if(evt->m_tinfo != NULL)
 				{
@@ -430,6 +454,9 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 	case PPME_SYSCALL_CHROOT_X:
 		parse_chroot_exit(evt);
 		break;
+	case PPME_SYSCALL_SETSID_X:
+		parse_setsid_exit(evt);
+		break;
 	default:
 		break;
 	}
@@ -441,20 +468,17 @@ void sinsp_parser::process_event(sinsp_evt *evt)
 #if defined(HAS_FILTERING) && defined(HAS_CAPTURE_FILTERING)
 	if(do_filter_later)
 	{
-		if(m_inspector->m_filter)
+		if(m_inspector->run_filters_on_evt(evt) == false)
 		{
-			if(m_inspector->m_filter->run(evt) == false)
-			{
-				evt->m_filtered_out = true;
-				return;
-			}
+			evt->m_filtered_out = true;
+			return;
 		}
 		evt->m_filtered_out = false;
 	}
 #endif
 	//
 	// Offline captures can produce events with the SCAP_DF_STATE_ONLY. They are
-	// supposed to go through the engine, but they must be filtered out before 
+	// supposed to go through the engine, but they must be filtered out before
 	// reaching the user.
 	//
 	if(!is_live)
@@ -666,7 +690,7 @@ bool sinsp_parser::reset(sinsp_evt *evt)
 			{
 				m_fd_listener->on_error(evt);
 			}
-			
+
 			if(evt->m_fdinfo->m_flags & sinsp_fdinfo_t::FLAGS_CLOSE_CANCELED)
 			{
 				//
@@ -885,7 +909,7 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		//
 		return;
 	}
-	
+
 	//
 	// Get the vtid to check if the clone is within a container
 	//
@@ -923,6 +947,58 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	{
 		//
 		// clone() returns 0 in the child.
+		//
+
+		int64_t parenttid;
+
+		//
+		// Check if this is a process or a new thread
+		//
+		if(flags & PPM_CL_CLONE_THREAD)
+		{
+			//
+			// This is a thread, the parent tid is the pid
+			//
+			parinfo = evt->get_param(4);
+			ASSERT(parinfo->m_len == sizeof(int64_t));
+			parenttid = *(int64_t *)parinfo->m_val;
+		}
+		else
+		{
+			//
+			// This is not a thread, the parent tid is ptid
+			//
+			parinfo = evt->get_param(5);
+			ASSERT(parinfo->m_len == sizeof(int64_t));
+			parenttid = *(int64_t *)parinfo->m_val;
+		}
+
+		//
+		// If the threadinfo in the event exists, and we're in
+		// a container, the threadinfo in the event must be
+		// stale (e.g. from a prior process with the same
+		// tid), because only the child side of a clone
+		// creates the threadinfo for the child. Clear and
+		// remove the old threadinfo.
+		//
+		if(evt->m_tinfo && in_container)
+		{
+			// See if the parent thread is in a
+			// container. If it is, the parent thread
+			// did *not* create the thread for this child,
+			// and any existing thread state must be
+			// stale.
+
+			sinsp_threadinfo* ptinfo = m_inspector->get_thread(parenttid, false, true);
+
+
+			if(ptinfo && ptinfo->m_tid != ptinfo->m_vtid)
+			{
+				m_inspector->remove_thread(tid, true);
+				evt->m_tinfo = NULL;
+			}
+		}
+
 		// Validate that the child thread info has actually been created.
 		//
 		if(!evt->m_tinfo)
@@ -945,27 +1021,7 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			//
 			childtid = tid;
 
-			//
-			// Check if this is a process or a new thread
-			//
-			if(flags & PPM_CL_CLONE_THREAD)
-			{
-				//
-				// This is a thread, the parent tid is the pid
-				//
-				parinfo = evt->get_param(4);
-				ASSERT(parinfo->m_len == sizeof(int64_t));
-				tid = *(int64_t *)parinfo->m_val;
-			}
-			else
-			{
-				//
-				// This is not a thread, the parent tid is ptid
-				//
-				parinfo = evt->get_param(5);
-				ASSERT(parinfo->m_len == sizeof(int64_t));
-				tid = *(int64_t *)parinfo->m_val;
-			}
+			tid = parenttid;
 
 			//
 			// Keep going and add the event with the standard code below
@@ -1069,17 +1125,20 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 
 		// Copy the root from the parent
 		tinfo.m_root = ptinfo->m_root;
+
+		// Copy the session id from the parent
+		tinfo.m_sid = ptinfo->m_sid;
 	}
 	else
 	{
 		//
-		// Parent is an invalid thread, which is strange since it's performing 
+		// Parent is an invalid thread, which is strange since it's performing
 		// a clone. We try to remove and look it up in proc.
 		//
 		m_inspector->remove_thread(tid, true);
 		tid_collision = true;
 
-		ptinfo = m_inspector->get_thread(tid, 
+		ptinfo = m_inspector->get_thread(tid,
 			true, true);
 
 		if(ptinfo == NULL)
@@ -1100,11 +1159,13 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 			tinfo.m_exe = ptinfo->m_exe;
 			tinfo.m_args = ptinfo->m_args;
 			tinfo.m_root = ptinfo->m_root;
+			tinfo.m_sid = ptinfo->m_sid;
 		}
 		else
 		{
 			//
-			// Parent not found in proc, use the event data
+			// Parent not found in proc, use the event data.
+			// (The session id will remain unset)
 			//
 			parinfo = evt->get_param(1);
 			tinfo.m_exe = (char*)parinfo->m_val;
@@ -1160,15 +1221,15 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		tinfo.m_pid = childtid;
 	}
 
-	//
-	// Copy the fd list
-	// XXX this is a gross oversimplification that will need to be fixed.
-	// What we do is: if the child is NOT a thread, we copy all the parent fds.
-	// The right thing to do is looking at PPM_CL_CLONE_FILES, but there are
-	// syscalls like open and pipe2 that can override PPM_CL_CLONE_FILES with the O_CLOEXEC flag
-	//
 	if(!(tinfo.m_flags & PPM_CL_CLONE_THREAD))
 	{
+		//
+		// Copy the fd list
+		// XXX this is a gross oversimplification that will need to be fixed.
+		// What we do is: if the child is NOT a thread, we copy all the parent fds.
+		// The right thing to do is looking at PPM_CL_CLONE_FILES, but there are
+		// syscalls like open and pipe2 that can override PPM_CL_CLONE_FILES with the O_CLOEXEC flag
+		//
 		tinfo.m_fdtable = *(ptinfo->get_fd_table());
 
 		//
@@ -1176,6 +1237,11 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		// referring to an element in the parent's table.
 		//
 		tinfo.m_fdtable.reset_cache();
+
+		//
+		// Not a thread, copy cwd
+		//
+		tinfo.m_cwd = ptinfo->m_cwd;
 	}
 	//if((tinfo.m_flags & (PPM_CL_CLONE_FILES)))
 	//{
@@ -1216,10 +1282,6 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	parinfo = evt->get_param(2);
 	tinfo.set_args(parinfo->m_val, parinfo->m_len);
 
-	// Copy the working directory
-	parinfo = evt->get_param(6);
-	tinfo.set_cwd(parinfo->m_val, parinfo->m_len);
-
 	// Copy the fdlimit
 	parinfo = evt->get_param(7);
 	ASSERT(parinfo->m_len == sizeof(int64_t));
@@ -1346,6 +1408,8 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	//
 	switch(etype)
 	{
+		case PPME_SYSCALL_FORK_20_X:
+		case PPME_SYSCALL_VFORK_20_X:
 		case PPME_SYSCALL_CLONE_20_X:
 			parinfo = evt->get_param(14);
 			tinfo.set_cgroups(parinfo->m_val, parinfo->m_len);
@@ -1363,6 +1427,14 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 	//
 	m_inspector->add_thread(tinfo);
 
+	//
+	// If there's a listener, invoke it
+	//
+	if(m_fd_listener)
+	{
+		m_fd_listener->on_clone(evt, &tinfo);
+	}
+
 	//
 	// If we had to erase a previous entry for this tid and rebalance the table,
 	// make sure we reinitialize the tinfo pointer for this event, as the thread
@@ -1375,8 +1447,8 @@ void sinsp_parser::parse_clone_exit(sinsp_evt *evt)
 		m_inspector->m_tid_collisions.push_back(tinfo.m_tid);
 #endif
 #ifdef _DEBUG
-		g_logger.format(sinsp_logger::SEV_INFO, 
-			"tid collision for %" PRIu64 "(%s)", 
+		g_logger.format(sinsp_logger::SEV_INFO,
+			"tid collision for %" PRIu64 "(%s)",
 			tinfo.m_tid, tinfo.m_comm.c_str());
 #endif
 	}
@@ -1446,10 +1518,6 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 	ASSERT(parinfo->m_len == sizeof(uint64_t));
 	evt->m_tinfo->m_pid = *(uint64_t *)parinfo->m_val;
 
-	// Get the working directory
-	parinfo = evt->get_param(6);
-	evt->m_tinfo->set_cwd(parinfo->m_val, parinfo->m_len);
-
 	// Get the fdlimit
 	parinfo = evt->get_param(7);
 	ASSERT(parinfo->m_len == sizeof(int64_t));
@@ -1556,6 +1624,15 @@ void sinsp_parser::parse_execve_exit(sinsp_evt *evt)
 #ifdef HAS_ANALYZER
 	evt->m_tinfo->m_ainfo->clear_role_flags();
 #endif
+
+	//
+	// If there's a listener, invoke it
+	//
+	if(m_fd_listener)
+	{
+		m_fd_listener->on_execve(evt);
+	}
+
 	return;
 }
 
@@ -1799,14 +1876,14 @@ void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 	{
 		//
 		// Populate the new fdi
-		//
+		//	
 		if(flags & PPM_O_DIRECTORY)
 		{
 			fdi.m_type = SCAP_FD_DIRECTORY;
 		}
 		else
 		{
-			fdi.m_type = SCAP_FD_FILE;		
+			fdi.m_type = SCAP_FD_FILE;
 		}
 
 		fdi.m_openflags = flags;
@@ -1817,7 +1894,11 @@ void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 		//
 		if(fdi.m_name == USER_EVT_DEVICE_NAME)
 		{
-			fdi.m_flags |= sinsp_fdinfo_t::FLAGS_IS_TRACER_FD;
+			fdi.m_flags |= sinsp_fdinfo_t::FLAGS_IS_TRACER_FILE;
+		}
+		else
+		{
+			fdi.m_flags |= sinsp_fdinfo_t::FLAGS_IS_NOT_TRACER_FD;
 		}
 
 		//
@@ -1837,7 +1918,7 @@ void sinsp_parser::parse_open_openat_creat_exit(sinsp_evt *evt)
 
 	if(m_fd_listener && !(flags & PPM_O_DIRECTORY))
 	{
-		m_fd_listener->on_file_create(evt, fullpath);
+		m_fd_listener->on_file_open(evt, fullpath, flags);
 	}
 }
 
@@ -2042,6 +2123,14 @@ void sinsp_parser::parse_bind_exit(sinsp_evt *evt)
 	// Update the name of this socket
 	//
 	evt->m_fdinfo->m_name = evt->get_param_as_str(1, &parstr, sinsp_evt::PF_SIMPLE);
+
+	//
+	// If there's a listener callback, invoke it
+	//
+	if(m_fd_listener)
+	{
+		m_fd_listener->on_bind(evt);
+	}
 }
 
 void sinsp_parser::parse_connect_exit(sinsp_evt *evt)
@@ -2108,6 +2197,8 @@ void sinsp_parser::parse_connect_exit(sinsp_evt *evt)
 
 			if(!(sinsp_utils::is_ipv4_mapped_ipv6(sip) && sinsp_utils::is_ipv4_mapped_ipv6(dip)))
 			{
+				evt->m_fdinfo->m_name = evt->get_param_as_str(1, &parstr, sinsp_evt::PF_SIMPLE);
+				evt->m_fdinfo->m_type = SCAP_FD_IPV6_SOCK;
 				return;
 			}
 
@@ -2737,7 +2828,7 @@ void sinsp_parser::swap_ipv4_addresses(sinsp_fdinfo_t* fdinfo)
 	fdinfo->m_sockinfo.m_ipv4info.m_fields.m_dport = tport;
 }
 
-void sinsp_parser::parse_tracer(sinsp_evt *evt, int64_t retval)
+uint32_t sinsp_parser::parse_tracer(sinsp_evt *evt, int64_t retval)
 {
 	sinsp_threadinfo* tinfo = evt->m_tinfo;
 	ASSERT(tinfo);
@@ -2765,7 +2856,8 @@ void sinsp_parser::parse_tracer(sinsp_evt *evt, int64_t retval)
 		{
 			evt->m_filtered_out = true;
 		}
-		return;
+
+		return p->m_res;
 	}
 
 	p->m_args.first = &p->m_argnames;
@@ -2800,6 +2892,13 @@ void sinsp_parser::parse_tracer(sinsp_evt *evt, int64_t retval)
 	}
 	else
 	{
+		uint32_t flags = evt->m_fdinfo->m_flags;
+
+		if(!(flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD))
+		{
+			return p->m_res;
+		}
+
 		//
 		// Parsing error.
 		// We don't know the direction, so we use enter.
@@ -2836,7 +2935,65 @@ void sinsp_parser::parse_tracer(sinsp_evt *evt, int64_t retval)
 	tinfo->m_latency = 0;
 	tinfo->m_last_latency_entertime = 0;
 
-	return;
+	return p->m_res;
+}
+
+bool sinsp_parser::detect_and_process_tracer_write(sinsp_evt *evt,
+	int64_t retval,
+	ppm_event_flags eflags)
+{
+	//
+	// Tracers get into the engine as normal writes, but the FD has a flag to
+	// quickly recognize them.
+	//
+	uint32_t flags = evt->m_fdinfo->m_flags;
+
+	if(!(flags & sinsp_fdinfo_t::FLAGS_IS_NOT_TRACER_FD))
+	{
+		sinsp_fdinfo_t* orifdinfo = evt->m_fdinfo;
+		if(orifdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD)
+		{
+			parse_tracer(evt, retval);
+			return true;
+		}
+		else
+		{
+			if(orifdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FILE)
+			{
+				if(eflags & EF_WRITES_TO_FD)
+				{
+					//
+					// We have not determined if this FD is a tracer FD or not.
+					// We're going to try to parse it.
+					// If the parsing succeeds, we mark it as a tracer FD. If it
+					// fails we mark it an NOT a tracer FD. Otherwise, we wait
+					// for the next buffer and we'll try again.
+					//
+					sinsp_tracerparser::parse_result pres =
+						(sinsp_tracerparser::parse_result)parse_tracer(evt, retval);
+
+					if(pres == sinsp_tracerparser::RES_OK)
+					{
+						//
+						// This FD has been recognized to be a tracer one.
+						// We do two things: mark it for future reference, and tell
+						// the driver to enable tracers capture (if we haven't done
+						// it yet).
+						//
+						orifdinfo->m_flags |= sinsp_fdinfo_t::FLAGS_IS_TRACER_FD;
+						m_inspector->enable_tracers_capture();
+						return true;
+					}
+					else if (pres == sinsp_tracerparser::RES_FAILED)
+					{
+						orifdinfo->m_flags |= sinsp_fdinfo_t::FLAGS_IS_NOT_TRACER_FD;
+					}
+				}
+			}
+		}
+	}
+
+	return false;
 }
 
 void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
@@ -2869,12 +3026,10 @@ void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
 	}
 
 	//
-	// User events get into the engine as normal writes, but the FD has a flag to
-	// quickly recognize them.
+	// Check if this is a tracer write on /dev/null, treat it in a special way
 	//
-	if(evt->m_fdinfo->m_flags & sinsp_fdinfo_t::FLAGS_IS_TRACER_FD)
+	if(detect_and_process_tracer_write(evt, retval, eflags))
 	{
-		parse_tracer(evt, retval);
 		return;
 	}
 
@@ -2963,7 +3118,7 @@ void sinsp_parser::parse_rw_exit(sinsp_evt *evt)
 			//
 			if(m_fd_listener)
 			{
-				m_fd_listener->on_read(evt, tid, evt->m_tinfo->m_lastevent_fd, evt->m_fdinfo, 
+				m_fd_listener->on_read(evt, tid, evt->m_tinfo->m_lastevent_fd, evt->m_fdinfo,
 					data, (uint32_t)retval, datalen);
 			}
 
@@ -3847,6 +4002,30 @@ void sinsp_parser::parse_container_json_evt(sinsp_evt *evt)
 		{
 			container_info.m_image = image.asString();
 		}
+		const Json::Value& imageid = container["imageid"];
+		if(!imageid.isNull() && imageid.isConvertibleTo(Json::stringValue))
+		{
+			container_info.m_imageid = imageid.asString();
+		}
+		const Json::Value& privileged = container["privileged"];
+		if(!privileged.isNull() && privileged.isConvertibleTo(Json::booleanValue))
+		{
+			container_info.m_privileged = privileged.asBool();
+		}
+
+		sinsp_container_info::parse_json_mounts(container["Mounts"], container_info.m_mounts);
+		const Json::Value& contip = container["ip"];
+		if(!contip.isNull() && contip.isConvertibleTo(Json::stringValue))
+		{
+			uint32_t ip;
+
+			if(inet_pton(AF_INET, contip.asString().c_str(), &ip) == -1)
+			{
+				throw sinsp_exception("Invalid 'ip' field while parsing container info: " + json);
+			}
+
+			container_info.m_container_ip = ntohl(ip);
+		}
 		const Json::Value& mesos_task_id = container["mesos_task_id"];
 		if(!mesos_task_id.isNull() && mesos_task_id.isConvertibleTo(Json::stringValue))
 		{
@@ -3863,7 +4042,7 @@ void sinsp_parser::parse_container_json_evt(sinsp_evt *evt)
 	}
 	else
 	{
-		g_logger.log("Invalid JSON encountered while parsing container info: " + json, sinsp_logger::SEV_ERROR);
+		throw sinsp_exception("Invalid JSON encountered while parsing container info: " + json);
 	}
 }
 
@@ -3912,6 +4091,42 @@ uint8_t* sinsp_parser::reserve_event_buffer()
 	}
 }
 
+int sinsp_parser::get_k8s_version(const std::string& json)
+{
+	if(m_k8s_capture_version == k8s_state_t::CAPTURE_VERSION_NONE)
+	{
+		g_logger.log(json, sinsp_logger::SEV_DEBUG);
+		Json::Value root;
+		if(Json::Reader().parse(json, root))
+		{
+			const Json::Value& items = root["items"]; // new
+			if(!items.isNull())
+			{
+				g_logger.log("K8s capture version " + std::to_string(k8s_state_t::CAPTURE_VERSION_2) + " detected.",
+							 sinsp_logger::SEV_DEBUG);
+				m_k8s_capture_version = k8s_state_t::CAPTURE_VERSION_2;
+				return m_k8s_capture_version;
+			}
+
+			const Json::Value& object = root["object"]; // old
+			if(!object.isNull())
+			{
+				g_logger.log("K8s capture version " + std::to_string(k8s_state_t::CAPTURE_VERSION_2) + " detected.",
+							 sinsp_logger::SEV_DEBUG);
+				m_k8s_capture_version = k8s_state_t::CAPTURE_VERSION_1;
+				return m_k8s_capture_version;
+			}
+			throw sinsp_exception("Unrecognized K8s capture format.");
+		}
+		else
+		{
+			throw sinsp_exception("Invalid K8s capture JSON encountered.");
+		}
+	}
+
+	return m_k8s_capture_version;
+}
+
 void sinsp_parser::parse_k8s_evt(sinsp_evt *evt)
 {
 	sinsp_evt_param *parinfo = evt->get_param(0);
@@ -3920,8 +4135,22 @@ void sinsp_parser::parse_k8s_evt(sinsp_evt *evt)
 	std::string json(parinfo->m_val, parinfo->m_len);
 	//g_logger.log(json, sinsp_logger::SEV_DEBUG);
 	ASSERT(m_inspector);
-	ASSERT(m_inspector->m_k8s_client);
-	m_inspector->m_k8s_client->simulate_watch_event(json);
+	if(!m_inspector)
+	{
+		throw sinsp_exception("Inspector is null, K8s client can not be created.");
+	}
+	if(!m_inspector->m_k8s_client)
+	{
+		m_inspector->make_k8s_client();
+	}
+	if(m_inspector->m_k8s_client)
+	{
+		m_inspector->m_k8s_client->simulate_watch_event(std::move(json), get_k8s_version(json));
+	}
+	else
+	{
+		throw sinsp_exception("K8s client can not be created.");
+	}
 }
 
 void sinsp_parser::parse_mesos_evt(sinsp_evt *evt)
@@ -3958,6 +4187,24 @@ void sinsp_parser::parse_chroot_exit(sinsp_evt *evt)
 	}
 }
 
+void sinsp_parser::parse_setsid_exit(sinsp_evt *evt)
+{
+	sinsp_evt_param *parinfo;
+	int64_t retval;
+
+	//
+	// Extract the return value
+	//
+	parinfo = evt->get_param(0);
+	retval = *(int64_t *)parinfo->m_val;
+	ASSERT(parinfo->m_len == sizeof(int64_t));
+
+	if(retval >= 0)
+	{
+		evt->get_thread_info()->m_sid = retval;
+	}
+}
+
 void sinsp_parser::free_event_buffer(uint8_t *ptr)
 {
 	if(m_tmp_events_buffer.size() < m_inspector->m_thread_manager->m_threadtable.size())
