@@ -72,6 +72,8 @@ public:
 	vector<sinsp_protodecoder*> m_open_callbacks;
 	vector<sinsp_protodecoder*> m_connect_callbacks;
 
+	ppm_event_flags m_drop_event_flags;
+
 	//
 	// Initializers
 	//
@@ -81,30 +83,31 @@ private:
 	//
 	// Initializers
 	//
-	void init_metaevt(metaevents_state& evt_state, uint16_t evt_type, uint16_t buf_size);
+	inline void init_metaevt(metaevents_state& evt_state, uint16_t evt_type, uint16_t buf_size);
 
 	//
 	// Helpers
 	//
 	bool reset(sinsp_evt *evt);
-	void store_event(sinsp_evt* evt);
+	inline void store_event(sinsp_evt* evt);
 
 	//
 	// Parsers
 	//
-	void parse_clone_exit(sinsp_evt* evt); 
-	void parse_execve_exit(sinsp_evt* evt); 
+	void parse_clone_exit(sinsp_evt* evt);
+	void parse_execve_exit(sinsp_evt* evt);
 	void proc_schedule_removal(sinsp_evt* evt);
-	void parse_open_openat_creat_exit(sinsp_evt* evt);	
+	void parse_open_openat_creat_exit(sinsp_evt* evt);
 	void parse_pipe_exit(sinsp_evt* evt);
 	void parse_socketpair_exit(sinsp_evt* evt);
-	void parse_socket_exit(sinsp_evt* evt);	
+	void parse_socket_exit(sinsp_evt* evt);
 	void parse_connect_exit(sinsp_evt* evt);
 	void parse_accept_exit(sinsp_evt* evt);
 	void parse_close_enter(sinsp_evt* evt);
 	void parse_close_exit(sinsp_evt* evt);
 	void parse_thread_exit(sinsp_evt* evt);
-	void parse_rw_exit(sinsp_evt* evt);
+	inline bool detect_and_process_tracer_write(sinsp_evt *evt, int64_t retval, ppm_event_flags eflags);
+	inline void parse_rw_exit(sinsp_evt* evt);
 	void parse_sendfile_exit(sinsp_evt* evt);
 	void parse_eventfd_exit(sinsp_evt* evt);
 	void parse_bind_exit(sinsp_evt* evt);
@@ -129,11 +132,13 @@ private:
 	void parse_setgid_exit(sinsp_evt* evt);
 	void parse_container_evt(sinsp_evt* evt); // deprecated, only for backward-compatibility
 	void parse_container_json_evt(sinsp_evt *evt);
-	inline void parse_tracer(sinsp_evt *evt, int64_t retval);
+	inline uint32_t parse_tracer(sinsp_evt *evt, int64_t retval);
 	void parse_cpu_hotplug_enter(sinsp_evt* evt);
+	int get_k8s_version(const std::string& json);
 	void parse_k8s_evt(sinsp_evt *evt);
 	void parse_chroot_exit(sinsp_evt *evt);
 	void parse_mesos_evt(sinsp_evt *evt);
+	void parse_setsid_exit(sinsp_evt *evt);
 
 	inline void add_socket(sinsp_evt* evt, int64_t fd, uint32_t domain, uint32_t type, uint32_t protocol);
 	inline void add_pipe(sinsp_evt *evt, int64_t tid, int64_t fd, uint64_t ino);
@@ -171,10 +176,13 @@ private:
 	vector<sinsp_protodecoder*> m_protodecoders;
 
 	metaevents_state m_k8s_metaevents_state;
+	int              m_k8s_capture_version = -1;
 	metaevents_state m_mesos_metaevents_state;
 
 	stack<uint8_t*> m_tmp_events_buffer;
 	friend class sinsp_analyzer;
 	friend class sinsp_analyzer_fd_listener;
 	friend class sinsp_protodecoder;
+	friend class sisnp_baseliner;
+	friend class sinsp_container_manager;
 };
