@@ -113,7 +113,12 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 //
 // Name of the device used for tracer injection
 //
-#define USER_EVT_DEVICE_NAME "/dev/sysdig"
+#define USER_EVT_DEVICE_NAME "/dev/null"
+
+//
+// For internal use
+//
+#define FALCOBL_FULL_PROCESSING
 
 //
 // FD class customized with the storage we need
