@@ -98,6 +98,7 @@ using namespace std;
 // The logger
 //
 extern sinsp_logger g_logger;
+#define lo g_logger.format
 
 //
 // Prototype of the callback invoked by the thread table when a thread is 
@@ -123,13 +124,17 @@ public:
 	virtual ~sinsp_fd_listener()
 	{
 	}
-	virtual void on_read(sinsp_evt *evt, int64_t tid, int64_t fd, sinsp_fdinfo_t* fdinfo, char *data, uint32_t original_len, uint32_t len) = 0;
-	virtual void on_write(sinsp_evt *evt, int64_t tid, int64_t fd, sinsp_fdinfo_t* fdinfo, char *data, uint32_t original_len, uint32_t len) = 0;
-	virtual void on_sendfile(sinsp_evt *evt, int64_t fdin, uint32_t len) = 0;
-	virtual void on_connect(sinsp_evt *evt, uint8_t* packed_data) = 0;
-	virtual void on_accept(sinsp_evt *evt, int64_t newfd, uint8_t* packed_data, sinsp_fdinfo_t* new_fdinfo) = 0;
-	virtual void on_file_create(sinsp_evt* evt, const string& fullpath) = 0;
+	virtual void on_read(sinsp_evt* evt, int64_t tid, int64_t fd, sinsp_fdinfo_t* fdinfo, char *data, uint32_t original_len, uint32_t len) = 0;
+	virtual void on_write(sinsp_evt* evt, int64_t tid, int64_t fd, sinsp_fdinfo_t* fdinfo, char *data, uint32_t original_len, uint32_t len) = 0;
+	virtual void on_sendfile(sinsp_evt* evt, int64_t fdin, uint32_t len) = 0;
+	virtual void on_connect(sinsp_evt* evt, uint8_t* packed_data) = 0;
+	virtual void on_accept(sinsp_evt* evt, int64_t newfd, uint8_t* packed_data, sinsp_fdinfo_t* new_fdinfo) = 0;
+	virtual void on_file_open(sinsp_evt* evt, const string& fullpath, uint32_t flags) = 0;
 	virtual void on_error(sinsp_evt* evt) = 0;
 	virtual void on_erase_fd(erase_fd_params* params) = 0;
 	virtual void on_socket_shutdown(sinsp_evt *evt) = 0;
+	virtual void on_execve(sinsp_evt* evt) = 0;
+	virtual void on_clone(sinsp_evt* evt, sinsp_threadinfo* newtinfo) = 0;
+	virtual void on_bind(sinsp_evt* evt) = 0;
+	virtual void on_new_container(const sinsp_container_info& container_info) = 0;
 };
