@@ -58,6 +58,7 @@ sinsp_threadinfo::sinsp_threadinfo(sinsp *inspector) :
 void sinsp_threadinfo::init()
 {
 	m_pid = (uint64_t) - 1LL;
+	m_sid = (uint64_t) - 1LL;
 	set_lastevent_data_validity(false);
 	m_lastevent_type = -1;
 	m_lastevent_ts = 0;
@@ -83,6 +84,7 @@ void sinsp_threadinfo::init()
 #endif
 	m_ainfo = NULL;
 	m_program_hash = 0;
+	m_program_hash_falco = 0;
 	m_lastevent_data = NULL;
 }
 
@@ -90,7 +92,7 @@ sinsp_threadinfo::~sinsp_threadinfo()
 {
 	uint32_t j;
 
-	if((m_inspector != NULL) && 
+	if((m_inspector != NULL) &&
 		(m_inspector->m_thread_manager != NULL) &&
 		(m_inspector->m_thread_manager->m_listener != NULL))
 	{
@@ -148,18 +150,53 @@ void sinsp_threadinfo::fix_sockets_coming_from_proc()
 	}
 }
 
+#define STR_AS_NUM_JAVA 0x6176616a
+#define STR_AS_NUM_RUBY 0x79627572
+#define STR_AS_NUM_PERL 0x6c726570
+#define STR_AS_NUM_NODE 0x65646f6e
+
 void sinsp_threadinfo::compute_program_hash()
 {
 	string phs = m_exe;
 
+	phs += m_container_id;
+
+	//
+	// By default, the falco hash is just exe+container
+	//
+	m_program_hash_falco = std::hash<std::string>()(phs);
+
+	//
+	// The program hash includes the arguments as well
+	//
 	for(auto arg = m_args.begin(); arg != m_args.end(); ++arg)
 	{
 		phs += *arg;
 	}
 
-	phs += m_container_id;
-
 	m_program_hash = std::hash<std::string>()(phs);
+
+	//
+	// For some specific processes (essentially the scripting languages)
+	// we include the arguments in the falco hash as well
+	//
+	if(m_comm.size() == 4)
+	{
+		uint32_t ncomm = *(uint32_t*)m_comm.c_str();
+
+		if(ncomm == STR_AS_NUM_JAVA || ncomm == STR_AS_NUM_RUBY ||
+			ncomm == STR_AS_NUM_PERL || ncomm == STR_AS_NUM_NODE)
+		{
+			m_program_hash_falco = m_program_hash;
+		}
+	}
+	else if(m_comm.size() >= 6)
+	{
+		if(m_comm.substr(0, 6) == "python")
+		{
+			m_program_hash_falco = m_program_hash;
+		}
+	}
 }
 
 void sinsp_threadinfo::add_fd_from_scap(scap_fdinfo *fdi, OUT sinsp_fdinfo_t *res)
@@ -190,7 +227,7 @@ void sinsp_threadinfo::add_fd_from_scap(scap_fdinfo *fdi, OUT sinsp_fdinfo_t *re
 		newfdi->m_sockinfo.m_ipv4serverinfo.m_port = fdi->info.ipv4serverinfo.port;
 		newfdi->m_sockinfo.m_ipv4serverinfo.m_l4proto = fdi->info.ipv4serverinfo.l4proto;
 		newfdi->m_name = ipv4serveraddr_to_string(&newfdi->m_sockinfo.m_ipv4serverinfo, m_inspector->m_hostname_and_port_resolution_enabled);
-			
+
 		//
 		// We keep note of all the host bound server ports.
 		// We'll need them later when patching connections direction.
@@ -199,7 +236,7 @@ void sinsp_threadinfo::add_fd_from_scap(scap_fdinfo *fdi, OUT sinsp_fdinfo_t *re
 
 		break;
 	case SCAP_FD_IPV6_SOCK:
-		if(sinsp_utils::is_ipv4_mapped_ipv6((uint8_t*)&fdi->info.ipv6info.sip) && 
+		if(sinsp_utils::is_ipv4_mapped_ipv6((uint8_t*)&fdi->info.ipv6info.sip) &&
 			sinsp_utils::is_ipv4_mapped_ipv6((uint8_t*)&fdi->info.ipv6info.dip))
 		{
 			//
@@ -264,7 +301,11 @@ void sinsp_threadinfo::add_fd_from_scap(scap_fdinfo *fdi, OUT sinsp_fdinfo_t *re
 
 		if(newfdi->m_name == USER_EVT_DEVICE_NAME)
 		{
-			newfdi->m_flags |= sinsp_fdinfo_t::FLAGS_IS_TRACER_FD;
+			newfdi->m_flags |= sinsp_fdinfo_t::FLAGS_IS_TRACER_FILE;
+		}
+		else
+		{
+			newfdi->m_flags |= sinsp_fdinfo_t::FLAGS_IS_NOT_TRACER_FD;
 		}
 
 		break;
@@ -280,7 +321,7 @@ void sinsp_threadinfo::add_fd_from_scap(scap_fdinfo *fdi, OUT sinsp_fdinfo_t *re
 	ASSERT(m_inspector != NULL);
 	vector<sinsp_protodecoder*>::iterator it;
 
-	for(it = m_inspector->m_parser->m_open_callbacks.begin(); 
+	for(it = m_inspector->m_parser->m_open_callbacks.begin();
 		it != m_inspector->m_parser->m_open_callbacks.end(); ++it)
 	{
 		(*it)->on_fd_from_proc(newfdi);
@@ -305,6 +346,7 @@ void sinsp_threadinfo::init(scap_threadinfo* pi)
 	m_tid = pi->tid;
 	m_pid = pi->pid;
 	m_ptid = pi->ptid;
+	m_sid = pi->sid;
 
 	m_comm = pi->comm;
 	m_exe = pi->exe;
@@ -493,6 +535,13 @@ void sinsp_threadinfo::set_cgroups(const char* cgroups, size_t len)
 		{
 			subsys = "memory";
 		}
+		else if(subsys == "io")
+		{
+			// blkio has been renamed just `io`
+			// in kernel space:
+			// https://github.com/torvalds/linux/commit/c165b3e3c7bb68c2ed55a5ac2623f030d01d9567
+			subsys = "blkio";
+		}
 
 		m_cgroups.push_back(std::make_pair(subsys, cgroup));
 		offset += subsys_length + 1 + cgroup.length() + 1;
@@ -554,7 +603,7 @@ bool sinsp_threadinfo::uses_client_port(uint16_t number)
 
 	sinsp_fdtable* fdt = get_fd_table();
 
-	for(it = fdt->m_table.begin(); 
+	for(it = fdt->m_table.begin();
 		it != fdt->m_table.end(); ++it)
 	{
 		if(it->second.m_type == SCAP_FD_IPV4_SOCK)
@@ -588,7 +637,12 @@ sinsp_threadinfo* sinsp_threadinfo::get_cwd_root()
 
 string sinsp_threadinfo::get_cwd()
 {
-	sinsp_threadinfo* tinfo = get_cwd_root();
+	// Ideally we should use get_cwd_root()
+	// but scap does not read CLONE_FS from /proc
+	// Also glibc and muslc use always 
+	// CLONE_THREAD|CLONE_FS so let's use
+	// get_main_thread() for now
+	sinsp_threadinfo* tinfo = get_main_thread();
 
 	if(tinfo)
 	{
@@ -604,15 +658,15 @@ string sinsp_threadinfo::get_cwd()
 void sinsp_threadinfo::set_cwd(const char* cwd, uint32_t cwdlen)
 {
 	char tpath[SCAP_MAX_PATH_SIZE];
-	sinsp_threadinfo* tinfo = get_cwd_root();
+	sinsp_threadinfo* tinfo = get_main_thread();
 
 	if(tinfo)
 	{
-		sinsp_utils::concatenate_paths(tpath, 
-			SCAP_MAX_PATH_SIZE, 
-			(char*)tinfo->m_cwd.c_str(), 
-			(uint32_t)tinfo->m_cwd.size(), 
-			cwd, 
+		sinsp_utils::concatenate_paths(tpath,
+			SCAP_MAX_PATH_SIZE,
+			(char*)tinfo->m_cwd.c_str(),
+			(uint32_t)tinfo->m_cwd.size(),
+			cwd,
 			cwdlen);
 
 		tinfo->m_cwd = tpath;
@@ -637,7 +691,7 @@ void sinsp_threadinfo::allocate_private_state()
 		m_private_state.clear();
 
 		vector<uint32_t>* sizes = &m_inspector->m_thread_privatestate_manager.m_memory_sizes;
-	
+
 		for(j = 0; j < sizes->size(); j++)
 		{
 			void* newbuf = malloc(sizes->at(j));
@@ -734,7 +788,7 @@ sinsp_threadinfo* sinsp_threadinfo::get_main_thread()
 			//
 			// No, this is either a single thread process or the root thread of a
 			// multithread process.
-			// Note: we don't set m_main_thread because there are cases in which this is 
+			// Note: we don't set m_main_thread because there are cases in which this is
 			//       invoked for a threadinfo that is in the stack. Caching the this pointer
 			//       would cause future mess.
 			//
@@ -941,7 +995,7 @@ void sinsp_thread_manager::remove_thread(threadinfo_map_iterator_t it, bool forc
 		//
 		// If the thread has a nonzero refcount, it means that we are forcing the removal
 		// of a main process or program that some childs refer to.
-		// We need to recalculate the child relationships, or the table will become 
+		// We need to recalculate the child relationships, or the table will become
 		// corrupted.
 		//
 		if(nchilds != 0)
@@ -1024,3 +1078,58 @@ void sinsp_thread_manager::update_statistics()
 	}
 #endif
 }
+
+void sinsp_thread_manager::to_scap()
+{
+/*
+	scap_proc_free_table(m_inspector->m_h);
+
+	for(auto it = m_threadtable.begin(); it != m_threadtable.end(); ++it)
+	{
+		sinsp_threadinfo& tinfo = it->second;
+
+		//
+		// Allocate the scap thread info
+		//
+		scap_threadinfo* sctinfo = (scap_threadinfo*)malloc(sizeof(scap_threadinfo));
+		if(sctinfo == NULL)
+		{
+			throw sinsp_exception("memory allocation error in sinsp_thread_manager::to_scap");
+		}
+
+		//
+		// Fill in the data
+		//
+sctinfo->tid = tinfo.m_tid;
+sctinfo->pid = tinfo.m_pid;
+sctinfo->ptid = tinfo.m_ptid;
+sctinfo->sid = tinfo.m_sid;
+
+strncpy(sctinfo->comm, tinfo.m_comm.c_str(), SCAP_MAX_PATH_SIZE);
+strncpy(sctinfo->exe, tinfo.m_exe.c_str(), SCAP_MAX_PATH_SIZE);
+tinfo.set_args(sctinfo->args, sctinfo->args_len);
+tinfo.set_env(sctinfo->env, sctinfo->env_len);
+tinfo.set_cwd(sctinfo->cwd, (uint32_t)strlen(sctinfo->cwd));
+sctinfo->flags = tinfo.m_flags ;
+sctinfo->fdlimit = tinfo.m_fdlimit;
+sctinfo->uid = tinfo.m_uid;
+sctinfo->gid = tinfo.m_gid;
+sctinfo->vmsize_kb = tinfo.m_vmsize_kb;
+sctinfo->vmrss_kb = tinfo.m_vmrss_kb;
+sctinfo->vmswap_kb = tinfo.m_vmswap_kb;
+sctinfo->pfmajor = tinfo.m_pfmajor;
+sctinfo->pfminor = tinfo.m_pfminor;
+sctinfo->vtid = tinfo.m_vtid;
+sctinfo->vpid = tinfo.m_vpid;
+
+tinfo.set_cgroups(sctinfo->cgroups, sctinfo->cgroups_len);
+sctinfo->root = tinfo.m_root;
+
+
+		//
+		// Add the created info to scap
+		//
+		scap_proc_add(m_inspector->m_h, it->second.m_tid, sctinfo);
+	}
+*/
+}
