@@ -37,7 +37,7 @@ typedef struct erase_fd_params
 	uint64_t m_ts;
 }erase_fd_params;
 
-/** @defgroup state State management 
+/** @defgroup state State management
  *  @{
  */
 
@@ -47,7 +47,7 @@ typedef struct erase_fd_params
   manipulate threads and retrieve thread information.
 
   \note As a library user, you won't need to construct thread objects. Rather,
-   you get them by calling \ref sinsp_evt::get_thread_info or 
+   you get them by calling \ref sinsp_evt::get_thread_info or
    \ref sinsp::get_thread.
   \note sinsp_threadinfo is also used to keep process state. For the sinsp
    library, a process is just a thread with TID=PID.
@@ -113,7 +113,7 @@ public:
 				//
 				// No, this is either a single thread process or the root thread of a
 				// multithread process.
-				// Note: we don't set m_main_thread because there are cases in which this is 
+				// Note: we don't set m_main_thread because there are cases in which this is
 				//       invoked for a threadinfo that is in the stack. Caching the this pointer
 				//       would cause future mess.
 				//
@@ -204,6 +204,7 @@ public:
 	int64_t m_tid;  ///< The id of this thread
 	int64_t m_pid; ///< The id of the process containing this thread. In single thread threads, this is equal to tid.
 	int64_t m_ptid; ///< The id of the process that started this thread.
+	int64_t m_sid; ///< The session id of the process containing this thread.
 	string m_comm; ///< Command name (e.g. "top")
 	string m_exe; ///< argv[0] (e.g. "sshd: user@pts/4")
 	vector<string> m_args; ///< Command line arguments (e.g. "-d1")
@@ -223,6 +224,8 @@ public:
 	int64_t m_vtid;  ///< The virtual id of this thread.
 	int64_t m_vpid; ///< The virtual id of the process containing this thread. In single thread threads, this is equal to vtid.
 	string m_root;
+	size_t m_program_hash;
+	size_t m_program_hash_falco;
 
 	//
 	// State for multi-event processing
@@ -230,7 +233,7 @@ public:
 	int64_t m_lastevent_fd; ///< The FD os the last event used by this thread.
 	uint64_t m_lastevent_ts; ///< timestamp of the last event for this thread.
 	uint64_t m_prevevent_ts; ///< timestamp of the event before the last for this thread.
-	uint64_t m_lastaccess_ts; ///< The last time this thread was looked up. Used when cleaning up the table. 
+	uint64_t m_lastaccess_ts; ///< The last time this thread was looked up. Used when cleaning up the table.
 	uint64_t m_clone_ts; ///< When the clone that started this process happened.
 
 	//
@@ -295,7 +298,7 @@ VISIBILITY_PRIVATE
 		else
 		{
 			m_lastevent_cpuid = (uint16_t) - 1;
-		}		
+		}
 	}
 	void allocate_private_state();
 	void compute_program_hash();
@@ -318,7 +321,6 @@ VISIBILITY_PRIVATE
 	uint16_t m_lastevent_type;
 	uint16_t m_lastevent_cpuid;
 	sinsp_evt::category m_lastevent_category;
-	size_t m_program_hash;
 
 	friend class sinsp;
 	friend class sinsp_parser;
@@ -330,6 +332,7 @@ VISIBILITY_PRIVATE
 	friend class thread_analyzer_info;
 	friend class sinsp_tracerparser;
 	friend class lua_cbacks;
+	friend class sisnp_baseliner;
 };
 
 /*@}*/
@@ -384,6 +387,8 @@ public:
 	void create_child_dependencies();
 	void recreate_child_dependencies();
 
+	void to_scap();
+
 	uint32_t get_thread_count()
 	{
 		return (uint32_t)m_threadtable.size();
@@ -423,4 +428,5 @@ private:
 	friend class sinsp_analyzer;
 	friend class sinsp;
 	friend class sinsp_threadinfo;
+	friend class sisnp_baseliner;
 };
