@@ -25,12 +25,17 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <sys/time.h>
 #include <netdb.h>
+#include <strings.h>
+#include <sys/ioctl.h>
+#include <fnmatch.h>
 #else
 #pragma comment(lib, "Ws2_32.lib")
 #include <WinSock2.h>
+#include "Shlwapi.h"
+#pragma comment(lib,"shlwapi.lib")
 #endif
-#include <algorithm> 
-#include <functional> 
+#include <algorithm>
+#include <functional>
 #include <errno.h>
 
 #include "sinsp.h"
@@ -41,7 +46,6 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "filterchecks.h"
 #include "chisel.h"
 #include "protodecoder.h"
-#include "json/json.h"
 #include "uri.h"
 #ifndef _WIN32
 #include "curl/curl.h"
@@ -64,19 +68,19 @@ const chiseldir_info g_chisel_dirs_array[] =
 #endif
 
 #ifndef _WIN32
-char* realpath_ex(const char *path, char *buff) 
+char* realpath_ex(const char *path, char *buff)
 {
-    char *home;
+	char *home;
 
-    if(*path=='~' && (home = getenv("HOME"))) 
-    {
-        char s[PATH_MAX];
-        return realpath(strcat(strcpy(s, home), path+1), buff);
-    } 
-    else 
-    {
-        return realpath(path, buff);
-    }
+	if(*path=='~' && (home = getenv("HOME")))
+	{
+		char s[PATH_MAX];
+		return realpath(strcat(strcpy(s, home), path+1), buff);
+		}
+	else
+	{
+		return realpath(path, buff);
+	}
 }
 #endif
 
@@ -112,7 +116,7 @@ sinsp_initializer::sinsp_initializer()
 	//
 	// Init the logger
 	//
-	g_logger.set_severity(sinsp_logger::SEV_DEBUG);
+	g_logger.set_severity(sinsp_logger::SEV_TRACE);
 
 #ifdef HAS_CHISELS
 	//
@@ -455,7 +459,7 @@ bool sinsp_utils::sockinfo_to_str(sinsp_sockinfo* sinfo, scap_fd_type stype, cha
 			{
 				char srcstr[INET6_ADDRSTRLEN];
 				char dststr[INET6_ADDRSTRLEN];
-				if(inet_ntop(AF_INET6, sip6, srcstr, sizeof(srcstr)) && 
+				if(inet_ntop(AF_INET6, sip6, srcstr, sizeof(srcstr)) &&
 					inet_ntop(AF_INET6, sip6, dststr, sizeof(dststr)))
 				{
 					snprintf(targetbuf,
@@ -491,7 +495,7 @@ bool sinsp_utils::sockinfo_to_str(sinsp_sockinfo* sinfo, scap_fd_type stype, cha
 			{
 				char srcstr[INET6_ADDRSTRLEN];
 				char dststr[INET6_ADDRSTRLEN];
-				if(inet_ntop(AF_INET6, sip6, srcstr, sizeof(srcstr)) && 
+				if(inet_ntop(AF_INET6, sip6, srcstr, sizeof(srcstr)) &&
 					inet_ntop(AF_INET6, sip6, dststr, sizeof(dststr)))
 				{
 					snprintf(targetbuf,
@@ -538,10 +542,10 @@ void rewind_to_parent_path(char* targetbase, char** tc, const char** pc, uint32_
 
 //
 // Args:
-//  - target: the string where we are supposed to start copying 
-//  - targetbase: the base of the path, i.e. the furthest we can go back when 
-//                following parent directories 
-//  - path: the path to copy 
+//  - target: the string where we are supposed to start copying
+//  - targetbase: the base of the path, i.e. the furthest we can go back when
+//                following parent directories
+//  - path: the path to copy
 //
 void copy_and_sanitize_path(char* target, char* targetbase, const char* path)
 {
@@ -661,11 +665,11 @@ void copy_and_sanitize_path(char* target, char* targetbase, const char* path)
 //
 // Return false if path2 is an absolute path
 //
-bool sinsp_utils::concatenate_paths(char* target, 
-									uint32_t targetlen, 
-									const char* path1, 
-									uint32_t len1, 
-									const char* path2, 
+bool sinsp_utils::concatenate_paths(char* target,
+									uint32_t targetlen,
+									const char* path1,
+									uint32_t len1,
+									const char* path2,
 									uint32_t len2)
 {
 	if(targetlen < (len1 + len2 + 1))
@@ -691,9 +695,13 @@ bool sinsp_utils::concatenate_paths(char* target,
 
 bool sinsp_utils::is_ipv4_mapped_ipv6(uint8_t* paddr)
 {
-	if(paddr[0] == 0 && paddr[1] == 0 && paddr[2] == 0 && paddr[3] == 0 && paddr[4] == 0 && 
-		paddr[5] == 0 && paddr[6] == 0 && paddr[7] == 0 && paddr[8] == 0 && paddr[9] == 0 && 
-		paddr[10] == 0xff && paddr[11] == 0xff)
+	if(paddr[0] == 0 && paddr[1] == 0 && paddr[2] == 0 && paddr[3] == 0 && paddr[4] == 0 &&
+		paddr[5] == 0 && paddr[6] == 0 && paddr[7] == 0 && paddr[8] == 0 && paddr[9] == 0 &&
+			(
+					( paddr[10] == 0xff && paddr[11] == 0xff) || // A real IPv4 address
+					(paddr[10] == 0 && paddr[11] == 0 && paddr[12] == 0 && paddr[13] == 0 && paddr[14] == 0 && paddr[15] == 0) // all zero address, assume IPv4 as well
+			)
+		)
 	{
 		return true;
 	}
@@ -718,7 +726,7 @@ const struct ppm_param_info* sinsp_utils::find_longest_matching_evt_param(string
 			const char* an = pi->name;
 			uint32_t alen = (uint32_t)strlen(an);
 			string subs = string(name, 0, alen);
-			
+
 			if(subs == an)
 			{
 				if(alen > maxlen)
@@ -748,6 +756,16 @@ uint64_t sinsp_utils::get_current_time_ns()
     return tv.tv_sec * (uint64_t) 1000000000 + tv.tv_usec * 1000;
 }
 
+bool sinsp_utils::glob_match(const char *pattern, const char *string)
+{
+#ifdef _WIN32
+	return PathMatchSpec(string, pattern) == TRUE;
+#else
+	int flags = 0;
+	return fnmatch(pattern, string, flags) == 0;
+#endif
+}
+
 #ifndef _WIN32
 void sinsp_utils::bt(void)
 {
@@ -762,7 +780,7 @@ void sinsp_utils::bt(void)
 	bt_size = backtrace(bt, 1024);
 	bt_syms = backtrace_symbols(bt, bt_size);
 	g_logger.format("%s", start);
-	for (i = 1; i < bt_size; i++) 
+	for (i = 1; i < bt_size; i++)
 	{
 		g_logger.format("%s", bt_syms[i]);
 	}
@@ -773,12 +791,41 @@ void sinsp_utils::bt(void)
 #endif // _WIN32
 
 ///////////////////////////////////////////////////////////////////////////////
-// gettimeofday() windows implementation
+// Time utility functions.
 ///////////////////////////////////////////////////////////////////////////////
+
+time_t get_epoch_utc_seconds(const std::string& time_str, const std::string& fmt)
+{
+#ifndef _WIN32
+	if(time_str.empty() || fmt.empty())
+	{
+		throw sinsp_exception("get_epoch_utc_seconds(): empty time or format string.");
+	}
+	struct tm tm_time = {0};
+	strptime(time_str.c_str(), fmt.c_str(), &tm_time);
+	tm_time.tm_isdst = -1; // strptime does not set this, signal timegm to determine DST
+	return timegm(&tm_time);
+#else
+	throw sinsp_exception("get_epoch_utc_seconds() not implemented on Windows");
+#endif // _WIN32
+}
+
+time_t get_epoch_utc_seconds_now()
+{
+#ifndef _WIN32
+	time_t rawtime;
+	time(&rawtime);
+	return timegm(gmtime(&rawtime));
+#else
+	throw sinsp_exception("get_now_seconds() not implemented on Windows");
+#endif // _WIN32
+}
+
+// gettimeofday() windows implementation
 #ifdef _WIN32
 
 #include <time.h>
-#include <windows.h> 
+#include <windows.h>
 
 const __int64 DELTA_EPOCH_IN_MICROSECS = 11644473600000000;
 
@@ -802,7 +849,7 @@ int gettimeofday(struct timeval *tv, struct timezone2 *tz)
 	// converting file time to unix epoch
 	//
 	tmpres /= 10;  // convert into microseconds
-	tmpres -= DELTA_EPOCH_IN_MICROSECS; 
+	tmpres -= DELTA_EPOCH_IN_MICROSECS;
 	tv->tv_sec = (__int32)(tmpres*0.000001);
 	tv->tv_usec =(tmpres%1000000);
 
@@ -884,13 +931,13 @@ string ipv4serveraddr_to_string(ipv4serverinfo* addr, bool resolve)
 	// IP address is saved with host byte order, that's why we do shifts
 	snprintf(buf,
 		sizeof(buf),
-		"%d.%d.%d.%d:%s", 
+		"%d.%d.%d.%d:%s",
 		(addr->m_ip & 0xFF),
 		((addr->m_ip & 0xFF00) >> 8),
 		((addr->m_ip & 0xFF0000) >> 16),
 		((addr->m_ip & 0xFF000000) >> 24),
 		port_to_string(addr->m_port, addr->m_l4proto, resolve).c_str());
-	
+
 	return string(buf);
 }
 
@@ -911,7 +958,7 @@ string ipv4tuple_to_string(ipv4tuple* tuple, bool resolve)
 	string dest = ipv4serveraddr_to_string(&info, resolve);
 
 	snprintf(buf, sizeof(buf), "%s->%s", source.c_str(), dest.c_str());
-	
+
 	return string(buf);
 }
 
@@ -928,7 +975,7 @@ string ipv6serveraddr_to_string(ipv6serverinfo* addr, bool resolve)
 	snprintf(buf,200,"%s:%s",
 		address,
 		port_to_string(addr->m_port, addr->m_l4proto, resolve).c_str());
-	
+
 	return string(buf);
 }
 
@@ -947,13 +994,13 @@ string ipv6tuple_to_string(_ipv6tuple* tuple, bool resolve)
 	{
 		return string();
 	}
-	
+
 	snprintf(buf,200,"%s:%s->%s:%s",
 		source_address,
 		port_to_string(tuple->m_fields.m_sport, tuple->m_fields.m_l4proto, resolve).c_str(),
 		destination_address,
 		port_to_string(tuple->m_fields.m_dport, tuple->m_fields.m_l4proto, resolve).c_str());
-	
+
 	return string(buf);
 }
 
@@ -965,14 +1012,14 @@ string ipv6tuple_to_string(_ipv6tuple* tuple, bool resolve)
 //
 vector<string> sinsp_split(const string &s, char delim)
 {
-    vector<string> res;
-    istringstream f(s);
-    string ts;
+	vector<string> res;
+	istringstream f(s);
+	string ts;
 
-    while(getline(f, ts, delim)) 
+	while(getline(f, ts, delim))
 	{
-        res.push_back(ts);
-    }
+		res.push_back(ts);
+	}
 
 	return res;
 }
@@ -980,7 +1027,7 @@ vector<string> sinsp_split(const string &s, char delim)
 //
 // trim from start
 //
-string& ltrim(string &s) 
+string& ltrim(string &s)
 {
 	s.erase(s.begin(), find_if(s.begin(), s.end(), not1(ptr_fun<int, int>(isspace))));
 	return s;
@@ -989,7 +1036,7 @@ string& ltrim(string &s)
 //
 // trim from end
 //
-string& rtrim(string &s) 
+string& rtrim(string &s)
 {
 	s.erase(find_if(s.rbegin(), s.rend(), not1(ptr_fun<int, int>(isspace))).base(), s.end());
 	return s;
@@ -998,93 +1045,85 @@ string& rtrim(string &s)
 //
 // trim from both ends
 //
-string& trim(string &s) 
+string& trim(string &s)
 {
 	return ltrim(rtrim(s));
 }
 
-void replace_in_place(string &s, const string &search, const string &replace)
+string& replace_in_place(string& str, const string& search, const string& replacement)
 {
-	for(size_t pos = 0; ; pos += replace.length()) 
+	string::size_type ssz = search.length();
+	string::size_type rsz = replacement.length();
+	string::size_type pos = 0;
+	while((pos = str.find(search, pos)) != string::npos)
 	{
-		// Locate the substring to replace
-		pos = s.find(search, pos);
-		if(pos == string::npos ) break;
-		// Replace by erasing and inserting
-		s.erase(pos, search.length());
-		s.insert(pos, replace );
+		str.replace(pos, ssz, replacement);
+		pos += rsz;
+		ASSERT(pos <= str.length());
 	}
+	return str;
 }
 
-void replace_in_place(string& str, string& substr_to_replace, string& new_substr) 
+string replace(const string& str, const string& search, const string& replacement)
 {
-	size_t index = 0;
-	uint32_t nsize = (uint32_t)substr_to_replace.size();
-
-	while (true) 
-	{
-		 index = str.find(substr_to_replace, index);
-		 if (index == string::npos) break;
-
-		 str.replace(index, nsize, new_substr);
-
-		 index += nsize;
-	}
+	string s(str);
+	replace_in_place(s, search, replacement);
+	return s;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_numparser implementation
 ///////////////////////////////////////////////////////////////////////////////
-uint32_t sinsp_numparser::parseu8(const string& str)
+uint8_t sinsp_numparser::parseu8(const string& str)
 {
 	uint32_t res;
 	char temp;
 
-	if(std::sscanf(str.c_str(), "%" PRIu8 "%c", &res, &temp) != 1)
+	if(std::sscanf(str.c_str(), "%" PRIu32 "%c", &res, &temp) != 1)
 	{
 		throw sinsp_exception(str + " is not a valid number");
 	}
 
-	return res;
+	return (uint8_t)res;
 }
 
-int32_t sinsp_numparser::parsed8(const string& str)
+int8_t sinsp_numparser::parsed8(const string& str)
 {
 	int32_t res;
 	char temp;
 
-	if(std::sscanf(str.c_str(), "%" PRId8 "%c", &res, &temp) != 1)
+	if(std::sscanf(str.c_str(), "%" PRId32 "%c", &res, &temp) != 1)
 	{
 		throw sinsp_exception(str + " is not a valid number");
 	}
 
-	return res;
+	return (int8_t)res;
 }
 
-uint32_t sinsp_numparser::parseu16(const string& str)
+uint16_t sinsp_numparser::parseu16(const string& str)
 {
 	uint32_t res;
 	char temp;
 
-	if(std::sscanf(str.c_str(), "%" PRIu16 "%c", &res, &temp) != 1)
+	if(std::sscanf(str.c_str(), "%" PRIu32 "%c", &res, &temp) != 1)
 	{
 		throw sinsp_exception(str + " is not a valid number");
 	}
 
-	return res;
+	return (uint16_t)res;
 }
 
-int32_t sinsp_numparser::parsed16(const string& str)
+int16_t sinsp_numparser::parsed16(const string& str)
 {
 	int32_t res;
 	char temp;
 
-	if(std::sscanf(str.c_str(), "%" PRId16 "%c", &res, &temp) != 1)
+	if(std::sscanf(str.c_str(), "%" PRId32 "%c", &res, &temp) != 1)
 	{
 		throw sinsp_exception(str + " is not a valid number");
 	}
 
-	return res;
+	return (int16_t)res;
 }
 
 uint32_t sinsp_numparser::parseu32(const string& str)
@@ -1239,124 +1278,32 @@ bool sinsp_numparser::tryparsed32_fast(const char* str, uint32_t strlen, int32_t
 // JSON helpers
 ///////////////////////////////////////////////////////////////////////////////
 
-std::string get_json_string(const Json::Value& root, const std::string& name)
+std::string get_json_string(const Json::Value& obj, const std::string& name)
 {
 	std::string ret;
-	Json::Value json_val = root[name];
-	if(!json_val.isNull() && json_val.isString())
+	const Json::Value& json_val = obj[name];
+	if(!json_val.isNull() && json_val.isConvertibleTo(Json::stringValue))
 	{
 		ret = json_val.asString();
 	}
 	return ret;
 }
 
-#if defined(__linux__)
 ///////////////////////////////////////////////////////////////////////////////
-// Curl helpers
+// socket helpers
 ///////////////////////////////////////////////////////////////////////////////
-sinsp_curl::sinsp_curl(const std::string& uristr, const std::string& cert): m_curl(curl_easy_init()), m_uri(new uri(uristr)), m_cert(cert), m_timeout(10)
+
+bool set_socket_blocking(int sock, bool block)
 {
-	if(!m_curl || !m_uri)
+#ifndef _WIN32
+	int arg = block ? 0 : 1;
+	if(ioctl(sock, FIONBIO, &arg) == -1)
+#else
+	u_long arg = block ? 0 : 1;
+	if(ioctlsocket(sock, FIONBIO, &arg) == -1)
+#endif // _WIN32
 	{
-		throw sinsp_exception("Cannot initialize CURL.");
+		return false;
 	}
+	return true;
 }
-
-sinsp_curl::~sinsp_curl()
-{
-	curl_easy_cleanup(m_curl);
-	delete m_uri;
-}
-
-string sinsp_curl::get_data()
-{
-	std::ostringstream os;
-	if(get_data(os))
-	{
-		return os.str();
-	}
-	return "";
-}
-
-bool sinsp_curl::get_data(std::ostream& os)
-{
-	CURLcode res = CURLE_OK;
-	curl_easy_setopt(m_curl, CURLOPT_URL, m_uri->to_string().c_str());
-	curl_easy_setopt(m_curl, CURLOPT_FOLLOWLOCATION, 1L);
-
-	if(m_uri->is_secure())
-	{
-		if(m_cert.empty())
-		{
-			check_error(curl_easy_setopt(m_curl, CURLOPT_SSL_VERIFYPEER , 0));
-		}
-		else
-		{
-			check_error(curl_easy_setopt(m_curl, CURLOPT_SSL_VERIFYPEER , 1));
-			res = curl_easy_setopt(m_curl, CURLOPT_CAINFO, m_cert.c_str());
-			if(res != CURLE_OK)
-			{
-				os << curl_easy_strerror(res) << std::flush;
-				return false;
-			}
-		}
-	}
-
-	curl_easy_setopt(m_curl, CURLOPT_NOSIGNAL, 1); //Prevent "longjmp causes uninitialized stack frame" bug
-	curl_easy_setopt(m_curl, CURLOPT_ACCEPT_ENCODING, "deflate");
-	curl_easy_setopt(m_curl, CURLOPT_WRITEFUNCTION, &sinsp_curl::write_data);
-	curl_easy_setopt(m_curl, CURLOPT_WRITEDATA, &os);
-
-	res = curl_easy_perform(m_curl);
-	if(res != CURLE_OK)
-	{
-		os << curl_easy_strerror(res) << std::flush;
-	}
-	else
-	{
-		// HTTP errors are not returned by curl API
-		// error will be in the response stream
-		long http_code = 0;
-		curl_easy_getinfo(m_curl, CURLINFO_RESPONSE_CODE, &http_code);
-		if(http_code >= 400)
-		{
-			return false;
-		}
-	}
-
-	return res == CURLE_OK;
-}
-
-void sinsp_curl::set_timeout(long seconds)
-{
-	m_timeout = seconds;
-}
-
-long sinsp_curl::get_timeout() const
-{
-	return m_timeout;
-}
-
-size_t sinsp_curl::write_data(void *ptr, size_t size, size_t nmemb, void *cb)
-{
-	std::string data(reinterpret_cast<const char*>(ptr), static_cast<size_t>(size * nmemb));
-	*reinterpret_cast<std::ostream*>(cb) << data << std::flush;
-	return size * nmemb;
-}
-
-void sinsp_curl::check_error(unsigned ret)
-{
-	if(ret >= CURL_LAST)
-	{
-		throw sinsp_exception("Invalid CURL return value:" + std::to_string(ret));
-	}
-
-	CURLcode res = (CURLcode)ret;
-	if(CURLE_OK != res && CURLE_AGAIN != res)
-	{
-		std::ostringstream os;
-		os << "Error: " << curl_easy_strerror(res);
-		throw sinsp_exception(os.str());
-	}
-}
-#endif // __linux__
