@@ -15,7 +15,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 --]]
 
 -- Chisel description
-description = "Shows the top TCP/UDP server ports in terms of total (in+out) bandwidth. This chisel is compatable with containers using the sysdig -pc or -pcontainer argument, otherwise no container information will be shown.";
+description = "Shows the top TCP/UDP server ports in terms of total (in+out) bandwidth. This chisel is compatible with containers using the sysdig -pc or -pcontainer argument, otherwise no container information will be shown.";
 short_description = "Top TCP/UDP server ports by R+W bytes";
 category = "Net";
 
