@@ -97,7 +97,7 @@ struct summary_table_entry_rsort_comparer
 //
 // Printer functions
 //
-void list_fields(bool verbose);
+void list_fields(bool verbose, bool markdown);
 const char* param_type_to_string(ppm_param_type pt);
 void list_events(sinsp* inspector);
 
